/* gwcc/src/callbacks.c
 *
 * Copyright (C) 2000, 2001, 2002 by Brent D. Ely
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

/* ---------------------------------------------------------------------------
 * Greetings From The Author:
 * ---------------------------------------------------------------------------
 * Hi there. Whether you are an Open Source advocate, curious academic,
 * a linux newbie, or Bill Gates himself (gag), you must surely recognize
 * the inherent power that lies in the ability to view the source code
 * of a program. Those of us who work in cubicles pounding out web
 * applications and Internet apps day-after-day are *used* to digging
 * through examples of HTML/Perl/JavaScripts. How can one really learn
 * to program without the ability to view other programs?
 *
 * Even in my CS undergrad classes we would have to run and analyze other
 * programs to really understand our own. And how is one supposed to continue
 * to learn without tutorials? I can view the API reference on
 * developer.gnome.org *all day long* and never know how i am supposed to
 * create and utilize preference files or setup/read a Save dialog!
 *
 * What about people with the abilty to visualize and code complex programs
 * but no access to a job at Oracle or Apple? How could one ever learn what
 * goes on inside programs like IE or Netscape (pre-mozilla)? We can all learn
 * to code simple C/C++ programs from books and tutorials, but how can me and
 * my friends ever hope to create the next Quake or SimCity without being able
 * to see and learn what goes on inside these projects and how to make such
 * monstrousities work?
 *
 * With the advent of Open Source - and in this case, GNOME - we can all
 * sharpen our learning curves dramatically. Being able to view the source
 * from programs that i find interesting really helps to bring my own projects
 * new strength and usability. Is this not what goes in on in science and
 * industry? If i want to build a bridge, i have to figure it all out myself?
 * No, there are open, accessable blue prints and diagrams for me to utilize.
 *
 * Bottom Line: i hope you find something in this program useful to your
 * next programming project. Some of my methods (preferences, for example)
 * may not conform to standard GNOME practices, but like, come on man,
 * coding is more of an art than a science! *lol*
 *
 * <soap box>
 *   Support Open Source (GPL et. al.) and contribute where you can,
 *   our children will be surfing the Net that we create today.
 * </soap box>
 *
 * Have fun and happy coding!
 * ..Brent (sfbrent@users.sourceforge.net)
 * <:-)
 * ---------------------------------------------------------------------------
 */

#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

/*--- Include Libraries ---*/
#include <gtk/gtk.h>            /* gtk_widget_queue_resize */
#include <gnome.h>
#include <signal.h>             /* for kill()             */
#include <sys/sysinfo.h>        /* for sysinfo()          */
//#include <syscall.h>          // Commented out for 0.9.8
//#include <linux/kernel.h>     // Commented out for 0.9.8
//#include <linux/sys.h>        // Commented out for 0.9.8
#include <netdb.h>              /* for gethostby*         */
#include <netinet/in.h>         /* IP Address structures  */
#include <arpa/inet.h>          /* for inet_ntoa()        */
#include <string.h>             /* for string funcs       */
#include <dirent.h>             /* for DIR and readdir    */
#include <time.h>               /* for gettimeofday       */
#include <sys/stat.h>           /* to stat(proc info)     */
#include <pwd.h>                /* pwd                    */
#include "callbacks.h"          /* the .h for this file   */
#include "prefs.h"              /* my PREFS functions     */
#include "interface.h"          /* GLADE-generated UI     */
#include "support.h"            /* GLADE-generated functs */
#include "utils.h"              /* my utility functions   */

/*--- Global Variables ---*/
   // Declare refs to app1 and others so we can interact with our widgets!
   extern GtkWidget * app1;         // The Main Application Window
   extern GtkWidget * proc_popmenu; // Popmenu - brought up by right-click in Process CList.
   GtkWidget * propertybox1;        // Dialog  - Preferences
   GtkWidget * dialog_print;        // Dialog  - "Print"
   int         pid2kill    = 0;
   int         signum      = 0;
   int         compare_col = 0;

   // Global Preference Variables
   extern char timestamp_str[80]; // The timestamp_string used in utils.c
   extern int  file_save_flag;
   extern int  timestamp_saved_file;
   extern char fixed_font[240];
   extern int  display_units;
   extern int  timestamp_scn_output;
   extern int  toolbar_setting_int;
   
   extern int  ifconfig_flag;
   extern int  df_flag_a;
   extern int  df_flag_h;
   extern int  df_flag_t;
   extern int  netstat_flag;

   extern int  grep_flag;
   extern int  use_grep_i_flag;
   extern int  omit_grep_sh_output;

   extern int  ping_c;
   extern int  ping_w;
   extern int  ping_i;
   extern int  use_traceroute_n;
   extern int  use_host_v;
   extern int  finger_flag;
   extern int  nslookup_retries;
   extern int  nslookup_timeout;


//***************************************************************************//
//  Function for Sorting Numeric Columns                                     //
//***************************************************************************//

static gint
compare(GtkCList *clist, gconstpointer p1, gconstpointer p2)
{
	GtkCell *cell1, *cell2;
	gchar   *fn1,   *fn2;

	cell1 = ((GtkCListRow *)p1)->cell;
	cell2 = ((GtkCListRow *)p2)->cell;

    // use the "+0" part to determine which column to get value from!
	fn1 = (cell1 + compare_col)->u.text;
	fn2 = (cell2 + compare_col)->u.text;

    if (atof(fn1) <= atof(fn2)) {
        return -1;
    }
    else {
        return 1;
    }
}


//***************************************************************************//
//  Program Functions...                                                     //
//***************************************************************************//

void
on_app1_destroy (GtkObject *object, gpointer user_data)
{
    // The user closed the GWCC window - lets kill the app now.
    on_exit1_activate(NULL, NULL);
}


// ------------------------------------------------------------------------- //
// ------------------------------- Menu Items ------------------------------ //
// ------------------------------------------------------------------------- //

void
on_save1_activate (GtkMenuItem *menuitem, gpointer user_data)
{
    // Call file saving function.
    if (perform_file_save("home", NULL) != 0) {
        gnome_dialog_run_and_close(GNOME_DIALOG(gnome_error_dialog
            ("No File Created!\nThere is nothing to save.")));
        return;
    }
}


void
on_save_as1_activate (GtkMenuItem *menuitem, gpointer user_data)
{
    // Get pointers to window widgets and declare local variables
    GtkWidget * file_dialog;

    // This opens the Standard GNOME File Selection dialog
    file_dialog = gtk_file_selection_new("Save As...");

    // Associate dialog Buttons with Handlers
    gtk_signal_connect(GTK_OBJECT(GTK_FILE_SELECTION(file_dialog)->ok_button),
                       "clicked",
                       GTK_SIGNAL_FUNC(on_button_save_as_okay_clicked),
                       file_dialog);

    gtk_signal_connect(GTK_OBJECT(GTK_FILE_SELECTION(file_dialog)->cancel_button),
                       "clicked",
                       GTK_SIGNAL_FUNC(on_button_save_as_cancel_clicked),
                       file_dialog);

    gtk_window_set_modal(GTK_WINDOW(file_dialog), TRUE);
    gtk_widget_show(file_dialog);
}


void
on_button_save_as_okay_clicked (GtkButton *button, gpointer user_data)
{
    // Get pointers to window widgets and declare local variables
    int result = 0;

    // Call GWCC file saving function (utils.c)
    result = perform_file_save("saveas", user_data);
    if (result == -2) {
        gnome_dialog_run_and_close(GNOME_DIALOG(gnome_error_dialog
            ("No File Created!\nYou must enter a filename to use Save As.")));
    }
    else if (result == -1) {
        gnome_dialog_run_and_close(GNOME_DIALOG(gnome_error_dialog
            ("No File Created!\nThere is nothing to save.")));
    }
}


void
on_button_save_as_cancel_clicked (GtkButton *button, gpointer user_data)
{
    // <Cancel> button was clicked, so close "Save As" Dialog
    gtk_widget_destroy(GTK_WIDGET(user_data));
}


void
on_print1_activate (GtkMenuItem *menuitem, gpointer user_data)
{
    // Create temp file to be Printed (via file_save function)
    // or ERROR out here (before even opening Print Dialog).
    if (perform_file_save("temp", NULL) != 0) {
        gnome_dialog_run_and_close(GNOME_DIALOG(gnome_error_dialog(
                                   "Operation Failed!\nThere is nothing to print.")));
        return;
    }

    // Create the "Print" Dialog
    dialog_print = create_dialog_print();
    gtk_widget_show(dialog_print);
}


void
on_exit1_activate (GtkMenuItem *menuitem, gpointer user_data)
{
    // Buh-bye!
    gtk_main_quit();
}


void
on_copy1_activate (GtkMenuItem *menuitem, gpointer user_data)
{
    // Get pointers to window widgets and declare local variables
    GtkWidget * notebook1     = lookup_widget(app1, "notebook1");
    GtkWidget * text_wsutils  = lookup_widget(app1, "text_wsutils");
    GtkWidget * entry_ipaddr  = lookup_widget(app1, "entry_ipaddr");
    GtkWidget * entry_to_copy = NULL;
    int notebook_page         = 0;

    // Get the number (int) of current notebook page (0...n-1)
    notebook_page = gtk_notebook_get_current_page((GtkNotebook *) notebook1);

    // ..then pick appropriate textbox to copy from.
    if (notebook_page == 0) {
        entry_to_copy = text_wsutils;
    }
    else if (notebook_page == 1) {
        entry_to_copy = lookup_widget(app1, "text_grep");
    }
    else if (notebook_page == 2) {
        entry_to_copy = lookup_widget(app1, "text_network");
    }
    else if (notebook_page == 3) {
        return;
    }
    else if (notebook_page == 4) {
    // brent - need method for determining which text to copy - 02/08/01
        gtk_entry_select_region(GTK_ENTRY(entry_ipaddr), 0,
                                strlen(gtk_entry_get_text(GTK_ENTRY(entry_ipaddr))));
        entry_to_copy = lookup_widget(app1, "entry_ipaddr");
    }
    gtk_editable_copy_clipboard(GTK_EDITABLE(entry_to_copy));
}


void
on_paste1_activate  (GtkMenuItem *menuitem, gpointer user_data)
{
    // Get pointers to window widgets and declare local variables
    GtkWidget * notebook1      = lookup_widget(app1, "notebook1");
    GtkWidget * entry_to_paste = NULL;
    int notebook_page          = 0;

    // Get the number (int) of current notebook page (0...n-1)
    // ..then pick appropriate textbox to copy from.
    notebook_page = gtk_notebook_get_current_page((GtkNotebook *) notebook1);

    if (notebook_page == 0) {
        return;
    }
    else if (notebook_page == 1) {
        entry_to_paste = lookup_widget(app1, "entry_network");
    }
    else if (notebook_page == 2) {
        entry_to_paste = lookup_widget(app1, "entry10");
    }
    else if (notebook_page == 3) {
	    return;
    }
    else if (notebook_page == 4) {
	    return;
    }

    gtk_editable_paste_clipboard(GTK_EDITABLE(entry_to_paste));
}


void
on_preferences1_activate (GtkMenuItem *menuitem, gpointer user_data)
{
    // Read prefs from disk and assign all global vars only done at startup.
    // read_prefs_from_disk() is called from main.c at startup, afterwords
    // all variables are set when user makes selection, then written to disk.

    // Create the Preferences PropertyBox
    propertybox1 = create_propertybox1();
    gtk_widget_show(propertybox1);

    // Fill correct values into Prefs Dialog Widgets across all pages.
    fill_prefs_dialog();
}


void
on_about1_activate (GtkMenuItem *menuitem, gpointer user_data)
{
    GtkWidget * about1;

    about1 = create_about1();
    gtk_widget_show(about1);
}


// ------------------------------------------------------------------------- //
// ------------------------------- Toolbar Buttons ------------------------- //
// ------------------------------------------------------------------------- //

void
on_button_save_clicked (GtkButton *button, gpointer user_data)
{
    on_save1_activate(NULL, user_data);
}


void
on_button_saveas_clicked (GtkButton *button, gpointer user_data)
{
    on_save_as1_activate(NULL, user_data);
}


void
on_button_copy_clicked (GtkButton *button, gpointer user_data)
{
    on_copy1_activate(NULL, user_data);
}


void
on_button_paste_clicked (GtkButton *button, gpointer user_data)
{
    on_paste1_activate(NULL, user_data);
}

void
on_button_print_clicked                (GtkButton       *button,
                                        gpointer         user_data)
{
    on_print1_activate(NULL, user_data);
}


// ------------------------------------------------------------------------- //
// ------------------------------ App1 Tab Items --------------------------- //
// ------------------------------------------------------------------------- //


void
on_grep_button_clicked (GtkButton *button, gpointer user_data)
{
    // Get pointers to window widgets and declare local variables
    GtkWidget * text_grep       = lookup_widget(app1, "text_grep");
    GtkWidget * entry10         = lookup_widget(app1, "entry10");
    GtkWidget * checkbutton20   = lookup_widget(app1, "checkbutton20");
    GtkWidget * checkbutton21   = lookup_widget(app1, "checkbutton21");
    GtkWidget * checkbutton22   = lookup_widget(app1, "checkbutton22");
    GdkFont   * fixed_font_ptr;
    FILE      * pipein_fp;
    char      * substr_ptr;
    char        grep_string[80] = "";
    char        readbuf[80]     = "";
    char        timestamp_str[80] = "";
    gchar     * grep_text;

    // Read grep text from textbox
    grep_text = gtk_entry_get_text((GtkEntry *) entry10);

    // PREFS: grep flags to use
    if (grep_flag == 0) {
       strcpy(grep_string, "ps | grep ");
    }
    else if (grep_flag == 1) {
       strcpy(grep_string, "ps -e | grep ");
    }
    else if (grep_flag == 2) {
       strcpy(grep_string, "ps -eH | grep ");
    }
    else if (grep_flag == 3) {
       strcpy(grep_string, "ps -efH | grep ");
    }
    // PREFS: use grep case insensitive (-i)
    if (use_grep_i_flag == 1) {
       strcat(grep_string, "-i ");
    }

    // Create final grep string to pass to system (add user string)
    strcat(grep_string, grep_text);

    // Read user output options and perform appropriate action.
    if (GTK_TOGGLE_BUTTON(checkbutton20)->active) {
        gtk_text_backward_delete(GTK_TEXT(text_grep), gtk_text_get_length(GTK_TEXT(text_grep)));
    }
    if (GTK_TOGGLE_BUTTON(checkbutton21)->active) {
        // PREFS: Fixed Font Size
        fixed_font_ptr = gdk_font_load((gchar *)fixed_font);
    }
    else {
        fixed_font_ptr = NULL;
    }

    // Create one way pipe to system and process command thru loop, then close pipe.
    if ((pipein_fp = popen(grep_string, "r")) == NULL) {
        perror("popen"); exit(1);
    }
    while (fgets(readbuf, 80, pipein_fp)) {
        // PREFS: omit grep shell command
        if (omit_grep_sh_output == 1) {
            substr_ptr = strstr(readbuf, "grep");
            if (!strlen((char *)&substr_ptr) >= 1) {
                gtk_text_insert(GTK_TEXT(text_grep), fixed_font_ptr, NULL, NULL, readbuf, -1);
            }
        }
        else {
            gtk_text_insert(GTK_TEXT(text_grep), fixed_font_ptr, NULL, NULL, readbuf, -1);
        }
    }
    pclose(pipein_fp);

    // PREFS: Timestamp Screen Output
    if (timestamp_scn_output) {
        strcpy(timestamp_str, get_timestamp() );
        gtk_text_insert(GTK_TEXT(text_grep), fixed_font_ptr, NULL, NULL, timestamp_str, -1);
    }

    // Insert divider after above output, if checkbutton clicked by client.
    if (GTK_TOGGLE_BUTTON(checkbutton22)->active) {
        gtk_text_insert(GTK_TEXT(text_grep), fixed_font_ptr, NULL, NULL,
          "-------------------------------------------------------------------------------- \n", -1);
    }
}


// ------------------------------------------------------------------------- //
// --->      All Buttons from NETWORK UTILS Notebook Page Follow!       <--- //
// ------------------------------------------------------------------------- //

/*****************************************************************************/
void
on_network_submit_clicked (GtkButton *button, gpointer user_data)
{
    // Get pointers to window widgets and declare local variables
    GtkWidget * checkbutton24  = lookup_widget(app1, "checkbutton24");
    GtkWidget * checkbutton25  = lookup_widget(app1, "checkbutton25");
    GtkWidget * checkbutton26  = lookup_widget(app1, "checkbutton26");
    GtkWidget * text_network   = lookup_widget(app1, "text_network");
    GtkWidget * entry6         = lookup_widget(app1, "entry6");
    GtkWidget * entry_network  = lookup_widget(app1, "entry_network");
    GtkWidget * gentry_network = lookup_widget(app1, "gentry_network");
    GdkFont   * fixed_font_ptr;
    FILE      * pipein_fp;
    gchar     * menu_text;
    gchar     * net_text;
    gchar       cmdline[80];
    char        readbuf[80];
    char        timestamp_str[80];
    int         done           = 0;
    int         count          = 0;

    // Read Text Boxes: TextEntry (host/ip) and OptionMenu (operation)
    menu_text = gtk_entry_get_text((GtkEntry *) entry6);
    net_text  = gtk_entry_get_text((GtkEntry *) entry_network);

    // Read user output options and perform appropriate action.
    if (GTK_TOGGLE_BUTTON(checkbutton24)->active) {
        gtk_text_backward_delete(GTK_TEXT(text_network), gtk_text_get_length(GTK_TEXT(text_network)));
    }
    if (GTK_TOGGLE_BUTTON(checkbutton25)->active) {
        // PREFS: Fixed Font Size
        fixed_font_ptr = gdk_font_load((gchar *)fixed_font);
    }
    else {
        fixed_font_ptr = NULL;
    }

    // In order to display STDERR messages, add "2>&1" to the end of command strings (nice!!)
    if (strncmp(menu_text, "ping", 4) == 0) {
        sprintf(cmdline, "ping -c %d -i %d -w %d %s 2>&1", ping_c, ping_i, ping_w, net_text);
        // NOTE: pinging some hosts creates a lock situation - 'ping -w' should prevent this,
        // but it doesnt seem to work on my linux box (or maybe at all?). -- 02/10/2001
    }
    else if (strncmp(menu_text, "whois", 5) == 0) {
        sprintf(cmdline, "whois %s 2>&1", net_text);
    }
    else if (strncmp(menu_text, "nslookup", 8) == 0) {
        sprintf(cmdline, "nslookup retry=%d timeout=%d %s 2>&1", nslookup_retries, nslookup_timeout, net_text);
    }
    else if (strncmp(menu_text, "traceroute", 10) == 0) {
        // PREFS: use traceroute -n flag
        if (use_traceroute_n) {
            sprintf(cmdline, "/usr/sbin/traceroute -n %s 2>&1", net_text);
        }
        else {
            sprintf(cmdline, "/usr/sbin/traceroute %s 2>&1", net_text);
        }
    }
    else if (strncmp(menu_text, "host", 4) == 0) {
        // PREFS: use host -v flag
        if (use_host_v) {
            sprintf(cmdline, "host -v %s 2>&1", net_text);
        }
        else {
            sprintf(cmdline, "host %s 2>&1", net_text);
        }
    }
    else if (strncmp(menu_text, "finger", 6) == 0) {
        // PREFS: Finger Flag
        if (finger_flag == 0) {
            sprintf(cmdline, "finger -s %s 2>&1", net_text);
        }
        else if (finger_flag == 1) {
            sprintf(cmdline, "finger -l %s 2>&1", net_text);
        }
    }
    else {
        sprintf(cmdline, "%s %s 2>&1", menu_text, net_text);
    }

    // Create one way pipe to system and process command thru loop, then close pipe.
    // Note: popen executes the command as a subprocess, then creates a pipe to it.
    if ((pipein_fp = popen(cmdline, "r")) == NULL) {
        perror("popen"); exit(1);
    }
    while (!done) {
        if (fgets(readbuf, 80, pipein_fp) != NULL) {
            gtk_text_insert(GTK_TEXT(text_network), fixed_font_ptr, NULL, NULL, readbuf, -1);
        }
        else if (readbuf != NULL) {
            done = TRUE;
        }
        else {
            sleep(2);
            count++;
        }

        if (count >= 3) {
            gtk_text_insert(GTK_TEXT(text_network), fixed_font_ptr, NULL, NULL,
                            "Operation Timed Out.\n", -1);
            done = TRUE;
        }
    }
    pclose(pipein_fp);

    // PREFS: Timestamp Screen Output
    if (timestamp_scn_output) {
        strcpy(timestamp_str, get_timestamp() );
        gtk_text_insert(GTK_TEXT(text_network), fixed_font_ptr, NULL, NULL, timestamp_str, -1);
    }

    // Insert divider after above output, if checkbutton clicked by client.
    if (GTK_TOGGLE_BUTTON(checkbutton26)->active) {
        gtk_text_insert(GTK_TEXT(text_network), fixed_font_ptr, NULL, NULL,
          "-------------------------------------------------------------------------------- \n", -1);
    }

    // Save users entry text to history list
    gnome_entry_append_history((GnomeEntry *) gentry_network, TRUE, (gchar*)net_text);
    gnome_entry_save_history((GnomeEntry *) gentry_network);
}


gboolean
on_entry_network_key_press_event (GtkWidget *widget, GdkEventKey *event, gpointer user_data)
{
    // If <RETURN> (65293) or <ENTER> (65421) key pressed, consider it a Button Press Event.
    if ((event->keyval == 65293) || (event->keyval == 65421)) {
        on_network_submit_clicked(NULL, NULL);
    }

    // Else return FALSE and keep going
    return FALSE;
}


// ------------------------------------------------------------------------- //
// --->     All Buttons from PROCESS CONTROL Notebook Page Follow!      <--- //
// ------------------------------------------------------------------------- //

void
on_signal_apply_clicked (GtkButton *button, gpointer user_data)
{
    // Get pointers to window widgets, declare local vars
    GtkWidget *signal_entry = lookup_widget(app1, "signal_entry");
//    GtkWidget *optionmenu_entry = lookup_widget(app1, "optionmenu_entry");
    gchar error_str[80];
    gchar *signal_name;

    // Get SIGNAL string from entry widget
    signal_name = gtk_entry_get_text(GTK_ENTRY(signal_entry));

    // Caluclate the signal number to send to system
    if (!strcmp(signal_name,"SIGHUP")) {
        signum = 1;
    }
    else if (!strcmp(signal_name,"SIGTERM")) {
        signum = 15;
    }
    else if (!strcmp(signal_name,"SIGKILL")) {
        signum = 9;
    }

    // Execute KILL Command
    // Show Error Dialog if we encounter problems killing process
    if (kill(pid2kill,signum) == -1) {
        sprintf(error_str, "OOPS! could not kill pid %d with signal %d!\n", pid2kill, signum);
        gnome_dialog_run_and_close(GNOME_DIALOG(gnome_error_dialog(error_str)));
        return;
    }

    // Reset signum
    signum = 0;

    // Refresh Process List
    on_process_refresh_clicked(NULL, NULL);
}


/*****************************************************************************/
void
on_process_refresh_clicked (GtkButton *button, gpointer user_data)
{
    // Props to the ps2 application that provided some insight on procs.
    // Get pointers to window widgets, declare local vars
    GtkWidget *process_clist      = lookup_widget(app1, "process_clist");
    GtkWidget *signal_apply       = lookup_widget(app1, "signal_apply");
    GtkWidget *combo6             = lookup_widget(app1, "combo6");
    GtkWidget *label_selected_pid = lookup_widget(app1, "label_selected_pid");
    struct     stat       finfo;
    struct     passwd    *pinfo;
    struct     dirent    *dinfo;
    struct     sysinfo    info;
    FILE      *pipein_fp;
    DIR       *dir;
    char      *dir_chr;
    char      *proc_str_list[6];
    char       mem_str[80];
    char       proc_file_name[256];
    char       p_state;
    char       st_buffer[128];
    char       st_buf2[32];
    char       st_buf3[32];
    char       st_buf4[32];
//    char       readbuf[80];
//    char       cpu_str[80];
    float      proc_cpu_usage;
    int        proc_total_cputime1; // will hold sum of utime & stime (user+system), used for %CPU
    int        proc_total_cputime2; // will hold sum of utime & stime (user+system), used for %CPU
    int        st_utime;           // # of jiffies sched in User Mode
    int        st_stime;           // # of jiffies sched in Kernel Mode
    int        st_starttime;
    int        st_vms;
    int        st_pri;
    int        p_parentpid;
    int        p_rss;
    int        total_ram;

//    int ps;
//    int rc;
//    int cpu,nice,system,idle;
//    int total_cpu;

/*
// On %CPU Calculation:
// ---------------------------------------------------------------------------------------
// From the LibGTop Reference (http://www.home-of-linux.org/gnome/libgtop)
// "Please note that all of the cpu values are absolute values measured in certain units
//  (to get seconds, divide them by frequency) since system boot. To get percentual values,
//  you need to call glibtop_cpu, save the result, wait some time and then call it again
//  and divide the differences of the two values by the time you have waited."
*/

// Get TOTAL_CPU and TOTAL_MEM
/*  TOO DAMN SLOW! - 05/01
    ps = open("/proc/stat", O_RDONLY);
    rc = read(ps, readbuf, sizeof(readbuf));
    close(ps);

    sscanf(readbuf, "%*s %d %d %d %d", &cpu, &nice, &system, &idle);
    total_cpu = cpu+nice+system+idle;
*/

    // Refreshing causes any selected row to be unselected, so disable Signal Button
    gtk_widget_set_sensitive((GtkWidget *) signal_apply, FALSE);
    gtk_widget_set_sensitive((GtkWidget *) combo6,       FALSE);

	// Clear clist & clear pid2kill value
    gtk_clist_clear(GTK_CLIST(process_clist));
    pid2kill = 0;

    // Clear and Disable gtk_text showing selected_pid
    gtk_label_set_text(GTK_LABEL(label_selected_pid), "N/A");
    gtk_widget_set_sensitive((GtkWidget *) label_selected_pid, FALSE);

    // Execute sysinfo() system call (fill struct with data).
    if (sysinfo(&info) != 0) {
        printf("ERROR: sysinfo call FAILED. \n");
        return;
    }

    // Set total_ram memory amount - used for %MEM below
    total_ram = (info.totalram / 1024);

    // Set column justifications
    gtk_clist_set_column_justification(GTK_CLIST(process_clist), 0, GTK_JUSTIFY_RIGHT);
    gtk_clist_set_column_justification(GTK_CLIST(process_clist), 1, GTK_JUSTIFY_CENTER);
    gtk_clist_set_column_justification(GTK_CLIST(process_clist), 2, GTK_JUSTIFY_RIGHT);
    gtk_clist_set_column_justification(GTK_CLIST(process_clist), 3, GTK_JUSTIFY_CENTER);
    gtk_clist_set_column_justification(GTK_CLIST(process_clist), 4, GTK_JUSTIFY_LEFT);
    gtk_clist_set_column_justification(GTK_CLIST(process_clist), 5, GTK_JUSTIFY_RIGHT);
//    gtk_clist_set_column_justification(GTK_CLIST(process_clist), 6, GTK_JUSTIFY_RIGHT);


    // Get %CPU and %MEM
//  too damn slow!
//    sprintf(proc_file_name, "ps --pid %s -uwh | sed -e 's/  */ /g' | cut -d' ' -f3,4,5 > /home/brent/test.txt", dinfo->d_name);
//    pipein_fp = popen(proc_file_name, "r");
//    pclose(pipein_fp);

    /* redirect stdin to /proc/meminfo */
/* free.c:
    if (open( "/proc/meminfo", O_RDONLY ) < 0) {
        perror("open");
        return 2;
    }
        fseek(stdin, 0L, SEEK_SET);
        fgets( buf1, 80, stdin );
*/


    // Parse all sub-dirs in proc for their stats using while loop
    dir = opendir("/proc/");
    while ((dinfo = readdir(dir)) != 0) {
        // Step through /proc sub-directory names one number at-a-time
        // find directories with names between 0-9 (processes).
        dir_chr = dinfo->d_name;
        while (*dir_chr) {
            if ((*dir_chr >= '0') && (*dir_chr <= '9')) {
                dir_chr++;
            }
            else {
                break;
            }
        }

        if (*dir_chr) {
            continue;
        }

        // Read each entry in the /proc directory
        strcpy(proc_file_name, "/proc/");
        strcat(proc_file_name, dinfo->d_name);
        strcat(proc_file_name, "/stat");

        // Open a pipe to the proc stat file and read info
        pipein_fp = fopen(proc_file_name, "r");
        fscanf(pipein_fp,
               "%*d %s %c %d %*d %*d %*d %*d %*u %*u %*u %*u %*u %d %d %*d %*d %*d %d %*u %*u %d %u %u",
               st_buffer,&p_state,&p_parentpid,&st_utime,&st_stime, &st_pri,&st_starttime,&st_vms,&p_rss);
        fclose(pipein_fp);

        // Get CPU Usage 1
        proc_total_cputime1 = ((float)st_utime + (float)st_stime);
        sleep(0.1);

        // Get CPU Usage 2
        pipein_fp = fopen(proc_file_name, "r");
        fscanf(pipein_fp,
               "%*d %s %c %d %*d %*d %*d %*d %*u %*u %*u %*u %*u %d %d %*d %*d %*d %d %*u %*u %d %u %u",
               st_buffer,&p_state,&p_parentpid,&st_utime,&st_stime, &st_pri,&st_starttime,&st_vms,&p_rss);
        fclose(pipein_fp);
        proc_total_cputime2 = ((float)st_utime + (float)st_stime);

        // Calculate %CPU for this proc
        proc_cpu_usage = ((proc_total_cputime2 - proc_total_cputime1) / 0.001);

/*
      // Following 2 lines from gpsui 2.2 app - 10/14/2001 - GOOD??
      temp = (float) (st_stime + st_utime) / (float) (uptime - st_starttime);
      cputime = temp * 100;
*/

//    printf("DEBUG: proc_total_cputime1=%d  proc_total_cputime2=%d \n", proc_total_cputime1, proc_total_cputime2);

        // Get process owner info
        stat(proc_file_name, &finfo);
        pinfo = getpwuid(finfo.st_uid);

        // Fill strings
        sprintf(st_buf2, "%c%c", p_state, p_rss ? ' ':'W');
        sprintf(st_buf3, "%d",   p_parentpid);
        sprintf(st_buf4, "%d",   p_rss); // p_rss = Memory Used

        // Get %CPU and %MEM
//        sprintf(proc_file_name, "ps --pid %s -uwh | sed -e 's/  */ /g' | cut -d' ' -f3,4,5 > /home/brent/test.txt", dinfo->d_name);
/*
        pipein_fp = popen(proc_file_name, "r");
        pclose(pipein_fp);

        if ((pipein_fp = fopen("/home/brent/test.txt", "r")) == NULL) {
            ferror("fopen"); exit(1);
        }
        while (fgets(readbuf, 80, pipein_fp)) {
            proc_str_list[0] = strtok(readbuf, " ");
            proc_str_list[5] = strtok(NULL, " ");
        }

        proc_str_list[6] = strtok(readbuf, " ");    // CPU%
        proc_str_list[5] = strtok(NULL, " ");       // MEM% // NOW CPU!!!
//    printf("DEBUG: PID=%s  CPU=%s  %d %d \n", dinfo->d_name, proc_str_list[5],st_utime,st_stime);
        pclose(pipein_fp);
*/

   // DEBUG ONLY!!!!
//   sprintf(proc_str_list[6], "%2.1f", ((100.0*(st_utime)) / 121000) ); //total_cpu));
//   sprintf(proc_str_list[6], "%2.1f", ((100.0*(st_utime+st_stime)) / 121000) ); //total_cpu));

//  printf("%d - %2.1f - %d \n", (100.0 * (float)p_rss / 94516), (100.0 * (float)p_rss / 94516), p_rss);
//bad   proc_str_list[5] = ((100 * (float)p_rss) / 94516); //total_mem);
//   proc_str_list[6] = dinfo->d_name;
   // DEBUG ^^^

        // Format/Construct %CPU and %MEM strings
        sprintf(mem_str, "%2.1f", ((100  * 4 * (float)p_rss) / total_ram));
//        sprintf(mem_str, "%2.1f", proc_cpu_usage);

// from 'ps' src   sprintf(cpu_str, "%2.1f", ((float)proc_total_cputime * 10 * 100/180)); // was 100/Hertz
// p->amount = total ? 100*(float)(p->user_time+p->kernel_time)/total : 0;

        // Add info to proc_clist one row at-a-time.
        proc_str_list[0] = dinfo->d_name;
        proc_str_list[1] = pinfo->pw_name;
        proc_str_list[2] = st_buf3;
        proc_str_list[3] = st_buf2;
        proc_str_list[4] = st_buffer;
        proc_str_list[5] = mem_str;

        // BRENT! - it would be cool to change color of text if proc_state = 'Z' - 04/24/01
        // "moving to 0.9.6" as the mozilla team is fond of saying.. :-)
//        gtk_clist_set_cell_style (GTK_CLIST(process_clist), gint row, gint column, GtkStyle *style);

        gtk_clist_append(GTK_CLIST(process_clist), proc_str_list);
    }
}

/*****************************************************************************/
void
on_process_clist_click_column (GtkCList  *clist,
                               gint       column,
                               gpointer   user_data)
{
    // Get pointers to window widgets, declare local vars
    GtkWidget *process_clist = lookup_widget(app1, "process_clist");
    static int current_col   = -1;

    // Set global var to use for compare function sorting.
    compare_col = column;

    // Set the column number to apply sort to
    gtk_clist_set_sort_column(GTK_CLIST(process_clist), column);

    // Choose sort type to pass to GTK (NULL = default, which is good for CHAR strings)
    // i wrote my own compare function per the GNOME docs to sort numeric columns.
    if ((column == 1) || (column == 3) || (column == 4)) {
        gtk_clist_set_compare_func(GTK_CLIST(process_clist), NULL);
    }
    else {
        gtk_clist_set_compare_func(clist, compare);
    }

    // Based upon current sort (ASC or DESC), choose sort order for this iteration.
    if (current_col == column) {
        gtk_clist_set_sort_type(GTK_CLIST(process_clist), GTK_SORT_DESCENDING);
        current_col = -1;
    }
    else {
        gtk_clist_set_sort_type(GTK_CLIST(process_clist), GTK_SORT_ASCENDING);
        current_col = column;
    }

    // Sort the column in process_clist widget
    gtk_clist_sort(clist);
}


/*****************************************************************************/
void
on_process_clist_select_row (GtkCList *clist,
                             gint      row,
                             gint      column,
                             GdkEvent *event,
                             gpointer  user_data)
{
    // Local Variables
    GtkWidget *signal_apply       = lookup_widget(app1, "signal_apply");
    GtkWidget *combo6             = lookup_widget(app1, "combo6");
    GtkWidget *label_selected_pid = lookup_widget(app1, "label_selected_pid");
    GtkWidget *frame52            = lookup_widget(app1, "frame52");
    gchar     *pid_text;

    // Get the contents of column 0 (pid) from user selected column
    gtk_clist_get_text(GTK_CLIST(clist), row, 0, &pid_text);

    // Set pid2kill global variable
    pid2kill = atoi(pid_text);

    // Clients cannot be allowed to kill INIT (gasp!), so disable <OK> button
    if (pid2kill == 1) {
        gtk_widget_set_sensitive((GtkWidget *)signal_apply, FALSE);
        gtk_widget_set_sensitive((GtkWidget *)combo6,       FALSE);
    }
    else {
        // Reactivate Signal Button if it was disabled previously
        gtk_widget_set_sensitive((GtkWidget *)signal_apply, TRUE);
        gtk_widget_set_sensitive((GtkWidget *)combo6,       TRUE);
    }

    // Set gtk_text showing selected pid, and resize frame
    gtk_label_set_text(GTK_LABEL(label_selected_pid), pid_text);
    gtk_widget_set_sensitive((GtkWidget *)label_selected_pid, TRUE);
    gtk_widget_queue_resize(GTK_WIDGET(frame52));
}


/*****************************************************************************/
void
on_process_clist_unselect_row (GtkCList        *clist,
                               gint             row,
                               gint             column,
                               GdkEvent        *event,
                               gpointer         user_data)
{
    // The row was unselected somehow (page flip?, refresh?) so clear pid2kill value.
    pid2kill = 0;
}


/*****************************************************************************/
gboolean
on_process_clist_button_press_event    (GtkWidget       *widget,
                                        GdkEventButton  *event,
                                        gpointer         user_data)
{
    // If button 3 (Right mouse click) then show popmenu with cool proc options!
    if (event->button == 3) {
        if (pid2kill >= 2) {
            gtk_menu_popup((GtkMenu *)proc_popmenu, NULL, NULL, NULL, NULL,
                           event->button, event->time);
        }
        return TRUE;
    }
	else {
        return FALSE;
    }
}


/*****************************************************************************/
void
on_send_sighup1_activate (GtkMenuItem *menuitem, gpointer user_data)
{
    signum = 1;
    on_signal_apply_clicked (NULL, NULL); //gpointer user_data)
}


void
on_send_sigterm1_activate  (GtkMenuItem *menuitem, gpointer user_data)
{
    signum = 15;
    on_signal_apply_clicked (NULL, NULL); //gpointer user_data)
}


void
on_send_sigkill1_activate  (GtkMenuItem *menuitem, gpointer user_data)
{
    signum = 9;
    on_signal_apply_clicked (NULL, NULL); //gpointer user_data)
}


void
on_get_detailed_info1_activate  (GtkMenuItem *menuitem, gpointer user_data)
{
    // BRENT - 05/20 - what are we going to do here?
}


// ------------------------------------------------------------------------- //
// --->    All Buttons from Workstation Utils Notebook Page Follow!     <--- //
// ------------------------------------------------------------------------- //

void
on_button_ifconfig_clicked (GtkButton *button, gpointer user_data)
{
    // Call function to perform operation and output to textbox.
    perform_wsutils_click("ifconfig");
}


void
on_button_netstat_clicked (GtkButton *button, gpointer user_data)
{
    // Call function to perform operation and output to textbox.
    perform_wsutils_click("netstat");
}


void
on_button_df_clicked (GtkButton *button, gpointer user_data)
{
    // Call function to perform operation and output to textbox.
    perform_wsutils_click("df");
}


void
on_button_path_clicked (GtkButton *button, gpointer user_data)
{
    // Call function to perform operation and output to textbox.
    perform_wsutils_click("path");
}


void
on_button_ldconfig_clicked (GtkButton *button, gpointer user_data)
{
    // Call function to perform operation and output to textbox.
    perform_wsutils_click("ldconfig");
}


void
on_button_lpq_clicked (GtkButton *button, gpointer user_data)
{
    // Call function to perform operation and output to textbox.
    perform_wsutils_click("lpq");
}


void
on_button_env_clicked (GtkButton *button, gpointer user_data)
{
    // Call function to perform operation and output to textbox.
    perform_wsutils_click("env");
}


void
on_button_w_clicked (GtkButton *button, gpointer user_data)
{
    // Call function to perform operation and output to textbox.
    perform_wsutils_click("w");
}


void
perform_wsutils_click (char *operation)
{
    // Get pointers to window widgets, declare local vars
    GtkWidget * text_wsutils  = lookup_widget(app1, "text_wsutils");
    GtkWidget * checkbutton16 = lookup_widget(app1, "checkbutton16");
    GtkWidget * checkbutton17 = lookup_widget(app1, "checkbutton17");
    GtkWidget * checkbutton18 = lookup_widget(app1, "checkbutton18");
    GdkFont   * fixed_font_ptr;
    FILE      * pipein_fp;
    char        readbuf[80]       = "";
    char        timestamp_str[80] = "";
    gchar       operation_str[80];

    // Read user output options and perform appropriate action.
    if (GTK_TOGGLE_BUTTON(checkbutton16)->active) {
        gtk_text_backward_delete(GTK_TEXT(text_wsutils), gtk_text_get_length(GTK_TEXT(text_wsutils)));
    }
    if (GTK_TOGGLE_BUTTON(checkbutton17)->active) {
        // PREFS: Fixed Font Size
        fixed_font_ptr = gdk_font_load((gchar *)fixed_font);
    }
    else {
        fixed_font_ptr = NULL;
    }

    // Construct full operation string - attach flags to base op as needed.
    if (strcmp(operation, "df") == 0) {
        sprintf(operation_str, "df");
        // PREFS: df flags
        if (df_flag_a) { strcat(operation_str, " -a"); }
        if (df_flag_h) { strcat(operation_str, " -h"); }
        if (df_flag_t) { strcat(operation_str, " -T"); }
    }
    else if (strcmp(operation, "ifconfig") == 0) {
        // PREFS: ifconfig flag
        strcpy(operation_str, "ifconfig 2>&1"); // use no flags for now!
// Brent - crap!, now RH7.2 doesnt allow use of ifconfig outside of 'su'!!!
// adding STDERR redirection so RH7.2 users can get feedback (12/28/2001)


    strcpy(operation_str, "netstat -a --inet -p 2>&1");


/* -- brent - CRAP! - 0.9.4 release testing (04/30) revealed the following:
   -- RedHat 7.1 bombs when using these flags, but
   -- they really arent great to begin with. (does anyone use these?! - i dont)
        if (ifconfig_flag == 0) { strcpy(operation_str, "ifconfig -arp"); }
        else if (ifconfig_flag == 1) { strcpy(operation_str, "ifconfig -promisc"); }
        else if (ifconfig_flag == 2) { strcpy(operation_str, "ifconfig -allmulti"); }
*/
    }
    else if (strcmp(operation, "path") == 0) {
        sprintf(operation_str, "echo $PATH");
    }
    else if (strcmp(operation, "netstat") == 0) {
        // PREFS: ifconfig flag
        if (netstat_flag == 0)      { strcpy(operation_str, "netstat -u 2>&1"); }
        else if (netstat_flag == 1) { strcpy(operation_str, "netstat -M 2>&1"); }
        else if (netstat_flag == 2) { strcpy(operation_str, "netstat -i 2>&1"); }
        else if (netstat_flag == 3) { strcpy(operation_str, "netstat -p 2>&1"); }
    }
    else {
        strcpy(operation_str, operation);
    }

    // Create one way pipe line with call to popen()
    if ((pipein_fp = popen(operation_str, "r")) == NULL) { perror("popen"); exit(1); }
    while(fgets(readbuf, 80, pipein_fp)) {
        gtk_text_insert(GTK_TEXT(text_wsutils), fixed_font_ptr, NULL, NULL, readbuf, -1);
    }
    pclose(pipein_fp);

    // PREFS: Timestamp Screen Output
    if (timestamp_scn_output) {
        strcpy(timestamp_str, get_timestamp() );
        gtk_text_insert(GTK_TEXT(text_wsutils), fixed_font_ptr, NULL, NULL, timestamp_str, -1);
    }

    // Insert divider after above output, if checkbutton clicked by client.
    if (GTK_TOGGLE_BUTTON(checkbutton18)->active) {
        gtk_text_insert(GTK_TEXT(text_wsutils), fixed_font_ptr, NULL, NULL,
          "-------------------------------------------------------------------------------- \n", -1);
    }
}


void
on_notebook1_switch_page (GtkNotebook     *notebook,
                          GtkNotebookPage *page,
                          gint             page_num,
                          gpointer         user_data)
{
    // Get pointers to window widgets, declare local vars
    GtkWidget *gentry_network = lookup_widget(app1, "gentry_network");
    GtkWidget *label_grep_ps  = lookup_widget(app1, "label_grep_ps");
    GtkWidget *save1          = lookup_widget(app1, "save1");
    GtkWidget *save_as1       = lookup_widget(app1, "save_as1");
    GtkWidget *print1         = lookup_widget(app1, "print1");
    GtkWidget *copy1          = lookup_widget(app1, "copy1");
    GtkWidget *paste1         = lookup_widget(app1, "paste1");
    GtkWidget *button_save    = lookup_widget(app1, "button_save");
    GtkWidget *button_saveas  = lookup_widget(app1, "button_saveas");
    GtkWidget *button_print   = lookup_widget(app1, "button_print");
    GtkWidget *button_copy    = lookup_widget(app1, "button_copy");
    GtkWidget *button_paste   = lookup_widget(app1, "button_paste");
    gchar      grep_label_str[80];

    // Network Utils Tab
    if (page_num == 1) {
        // Load history into Network Text GnomeEntry Widget
        gentry_network = lookup_widget(app1, "gentry_network");
        gnome_entry_load_history((GnomeEntry *) gentry_network);
    }
    // Process Grep Tab
    // Change gtk_label "ps | grep" to match prefs!
    else if (page_num == 2) {
        // Construct the first part of the grep label string
        if (grep_flag == 0) {
            strcpy(grep_label_str, "ps | grep ");
        }
        else if (grep_flag == 1) {
            strcpy(grep_label_str, "ps -e | grep ");
        }
        else if (grep_flag == 2) {
            strcpy(grep_label_str, "ps -eH | grep ");
        }
        else if (grep_flag == 3) {
            strcpy(grep_label_str, "ps -efH | grep ");
        }
        // Add `-i` flag if necessary
        if (use_grep_i_flag == 1) {
            strcat(grep_label_str, "-i ");
        }
        // Set the actual gtk_label to string value
        gtk_label_set_text(GTK_LABEL(label_grep_ps), grep_label_str);
    }
    // Process CList
    else if (page_num == 3) {
        gtk_widget_set_sensitive((GtkWidget *) save1,         FALSE);
        gtk_widget_set_sensitive((GtkWidget *) save_as1,      FALSE);
        gtk_widget_set_sensitive((GtkWidget *) print1,        FALSE);
        gtk_widget_set_sensitive((GtkWidget *) copy1,         FALSE);
        gtk_widget_set_sensitive((GtkWidget *) paste1,        FALSE);
        gtk_widget_set_sensitive((GtkWidget *) button_save,   FALSE);
        gtk_widget_set_sensitive((GtkWidget *) button_saveas, FALSE);
        gtk_widget_set_sensitive((GtkWidget *) button_print,  FALSE);
        gtk_widget_set_sensitive((GtkWidget *) button_copy,   FALSE);
        gtk_widget_set_sensitive((GtkWidget *) button_paste,  FALSE);

        on_process_refresh_clicked(NULL, NULL);
        return;
    }
    // System Stats Tab
    else if (page_num == 4) {
        gtk_widget_set_sensitive((GtkWidget *) save1,         FALSE);
        gtk_widget_set_sensitive((GtkWidget *) save_as1,      FALSE);
        gtk_widget_set_sensitive((GtkWidget *) print1,        FALSE);
        gtk_widget_set_sensitive((GtkWidget *) copy1,         TRUE);
        gtk_widget_set_sensitive((GtkWidget *) paste1,        FALSE);
        gtk_widget_set_sensitive((GtkWidget *) button_save,   FALSE);
        gtk_widget_set_sensitive((GtkWidget *) button_saveas, FALSE);
        gtk_widget_set_sensitive((GtkWidget *) button_print,  FALSE);
        gtk_widget_set_sensitive((GtkWidget *) button_copy,   TRUE);
        gtk_widget_set_sensitive((GtkWidget *) button_paste,  FALSE);

        update_sys_stats_tab(notebook, page, page_num, user_data);
        return;
    }

    // Reactivate buttons that may have become inactive before in other tabs.
    // The first two functs have no return statement so they do fall out here.
    gtk_widget_set_sensitive((GtkWidget *) save1,         TRUE);
    gtk_widget_set_sensitive((GtkWidget *) save_as1,      TRUE);
    gtk_widget_set_sensitive((GtkWidget *) print1,        TRUE);
    gtk_widget_set_sensitive((GtkWidget *) copy1,         TRUE);
    gtk_widget_set_sensitive((GtkWidget *) paste1,        TRUE);
    gtk_widget_set_sensitive((GtkWidget *) button_save,   TRUE);
    gtk_widget_set_sensitive((GtkWidget *) button_saveas, TRUE);
    gtk_widget_set_sensitive((GtkWidget *) button_copy,   TRUE);
    gtk_widget_set_sensitive((GtkWidget *) button_paste,  TRUE);
    gtk_widget_set_sensitive((GtkWidget *) button_print,  TRUE);

    // We've done our few items of work for a given tab, now go on.
    return;
}


void
update_sys_stats_tab (GtkNotebook     *notebook,
                      GtkNotebookPage *page,
                      gint             page_num,
                      gpointer         user_data)
{
    // Get pointers to window widgets, declare local vars
    GtkWidget *users_clist         = lookup_widget(app1, "users_clist");
    GtkWidget *entry_ipaddr        = lookup_widget(app1, "entry_ipaddr");
    GtkWidget *label_apache        = lookup_widget(app1, "label_apache");
    GtkWidget *label_samba         = lookup_widget(app1, "label_samba");
    GtkWidget *label_sendmail      = lookup_widget(app1, "label_sendmail");
    GtkWidget *label_uptime        = lookup_widget(app1, "label_uptime");
    GtkWidget *label_totalram      = lookup_widget(app1, "label_totalram");
    GtkWidget *label_totalswap     = lookup_widget(app1, "label_totalswap");
    GtkWidget *label_hardware      = lookup_widget(app1, "label_hardware");
    GtkWidget *label_kernel        = lookup_widget(app1, "label_kernel");
    GtkWidget *progressbar_mem     = lookup_widget(app1, "progressbar_mem");
    GtkWidget *progressbar_swap    = lookup_widget(app1, "progressbar_swap");
    GtkWidget *progressbar_sleep   = lookup_widget(app1, "progressbar_sleep");
    GtkWidget *progressbar_run     = lookup_widget(app1, "progressbar_run");
    GtkWidget *progressbar_zombie  = lookup_widget(app1, "progressbar_zombie");
    GtkWidget *progressbar_stopped = lookup_widget(app1, "progressbar_stopped");
    FILE      *pipein_fp;
    DIR       *dir;
    struct    dirent    *dinfo;
    struct    sysinfo    info;
//    struct    hostent   *hostentAnswer;
//    struct    in_addr   *netAddr;
    char      readbuf[1024];
    char      uptime_str[80]      = "";
    char      apache_str[80]      = "Not Running";
    char      sendmail_str[80]    = "Not Running";
    char      samba_str[80]       = "Not Running";
    char      users_str[1024]     = "";
    char      hardware_str[80]    = "";
    char      kernel_str[80]      = "";
    char     *hardware_cpu;
    char     *hardware_clock;
    char     *hardware_machine;
    char     *hardware_str_ptr;
    char     *kernel_str_ptr;
    char      totalram_str[80];
    char      totalswap_str[80];
//    char      hostname[256];
    char      IPaddress[32];
    char      proc_buffer[128];
    char      proc_file_name[256];
    char      proc_state;
    char     *dir_chr;
    char     *users_str_list[3] = {"user", "term", "time"};
    char     *tmp1;
    char     *tmp2;
    char     *tmp3;
    int       uptime_hours        = 0;
    int       uptime_days         = 0;
//    int       uptime_months       = 0;
    int       num_proc_total      = 0;
    int       num_proc_sleep      = 0;
    int       num_proc_run        = 0;
    int       num_proc_zombie     = 0;
    int       num_proc_stopped    = 0;
//    int       a                   = 0;
    gfloat    mem_percentage    = 0.0;
    gfloat    swap_percentage   = 0.0;
    extern int h_errno; // Comes from <netdb.h> - used for determining error code...IPaddr


    // Exee sysinfo() system call (fill struct with data).
    if (sysinfo(&info) != 0) {
        printf("ERROR: sysinfo call FAILED. \n");
        return;
    }

    // ------------------
    // Set Uptime TextBox
    // ------------------
    // 12/11 - these formulas are gay - why cant we employ UNIX-style or Oracle style (decimal pts)
    //         by reading /proc/* files?
    uptime_days   = (((info.uptime / 60) / 60) / 24);
    uptime_hours  = (((info.uptime / 60) / 60) - (uptime_days * 24));
//    uptime_months = (  uptime_days / 30  ); // help!
//    sprintf(uptime_str, "%d Months, %ld Days, %0.1f Hours", uptime_months, uptime_days, uptime_hours);
    sprintf(uptime_str, "%ld Days, %d Hours", (long)uptime_days, uptime_hours);
    gtk_label_set_text(GTK_LABEL(label_uptime), uptime_str);

    // --------------------
    // Set Free/Swap Memory
    // --------------------
    // PREFS: display units
    if (display_units == 0) {
        sprintf(totalram_str,  " %ld M ", info.totalram /1048576);
        sprintf(totalswap_str, " %ld M ", info.totalswap/1048576);
    }
    else if (display_units == 1) {
        sprintf(totalram_str,  " %ld k ", info.totalram /1024);
        sprintf(totalswap_str, " %ld k ", info.totalswap/1024);
    }
    gtk_label_set_text(GTK_LABEL(label_totalram),  totalram_str);
    gtk_label_set_text(GTK_LABEL(label_totalswap), totalswap_str);

    // ----------------------------------------
    // Set Memory ProgressBars (i love these!)
    // ----------------------------------------
    mem_percentage  = ( 1 - ((float)info.freeram  / (float)info.totalram));
    swap_percentage = ( 1 - ((float)info.freeswap / (float)info.totalswap));
    if (swap_percentage < 0) { swap_percentage = 0; } // Keep bar looking good
    gtk_progress_bar_update((GtkProgressBar *)progressbar_mem,  mem_percentage);
    gtk_progress_bar_update((GtkProgressBar *)progressbar_swap, swap_percentage);
/*
    gtk_progress_configure((GtkProgress *)progressbar_mem,
                           (gfloat)(info.totalram/1000000 - info.freeram/1000000),
                            0.0, (gfloat)info.totalram/1000000);
*/
// above is experimental (unused in 0.9.5) - 06/15/01

    // --------------------------------
    // ---> Fill Entry: Apache Status
    // --------------------------------
    gtk_label_set_text(GTK_LABEL(label_apache), apache_str);
    gtk_widget_set_sensitive((GtkWidget*)label_apache, FALSE);

    if ((pipein_fp = popen("ps -e | grep -i httpd", "r")) == NULL) {
        perror("popen"); exit(1);
    }
    while(fgets(readbuf, 1024, pipein_fp)) {
        if (strcmp(readbuf, "httpd")) {
            sprintf(apache_str, "Accepting Connections");
            gtk_widget_set_sensitive((GtkWidget*)label_apache, TRUE);
        }
    }
    gtk_label_set_text(GTK_LABEL(label_apache), apache_str);
    pclose(pipein_fp);

    // --------------------------------
    // ---> Fill Entry: Sendmail Status
    // --------------------------------
    gtk_label_set_text(GTK_LABEL(label_sendmail), sendmail_str);
    gtk_widget_set_sensitive((GtkWidget*)label_sendmail, FALSE);

    if ((pipein_fp = popen("ps -e | grep -i sendmail", "r")) == NULL) {
        perror("popen"); exit(1);
    }
    while(fgets(readbuf, 1024, pipein_fp)) {
        if (strcmp(readbuf, "accepting connections")) {
            sprintf(sendmail_str, "Accepting Connections");
            gtk_widget_set_sensitive((GtkWidget*)label_sendmail, TRUE);
        }
    }
    gtk_label_set_text(GTK_LABEL(label_sendmail), sendmail_str);
    pclose(pipein_fp);

    // --------------------------------
    // ---> Fill Entry: Samba Status
    // --------------------------------
    gtk_label_set_text(GTK_LABEL(label_samba), samba_str);
    gtk_widget_set_sensitive((GtkWidget*)label_samba, FALSE);

    if ((pipein_fp = popen("ps -e | grep -i smbd", "r")) == NULL) {
        perror("popen"); exit(1);
    }
    while (fgets(readbuf, 1024, pipein_fp)) {
        if (strcmp(readbuf, "accepting connections")) {
            sprintf(samba_str, "Accepting Connections");
            gtk_widget_set_sensitive((GtkWidget*)label_samba, TRUE);
        }
    }
    gtk_label_set_text(GTK_LABEL(label_samba), samba_str);
    pclose(pipein_fp);

    // --------------------------------
    // ---> Fill CList: Users
    // --------------------------------
    if ((pipein_fp = popen("who", "r")) == NULL) {
        perror("popen"); exit(1);
    }
    gtk_clist_clear(GTK_CLIST(users_clist));
    while (fgets(readbuf, 1024, pipein_fp)) {
        strcpy(users_str, readbuf);
        users_str_list[0] = strtok(users_str, " ");
        users_str_list[1] = strtok(NULL, " ");
        tmp1 = strtok(NULL, " ");
        tmp2 = strtok(NULL, " ");
        tmp3 = strtok(NULL, " ");
        sprintf(readbuf, "%s %s %s", tmp1, tmp2, tmp3);
        users_str_list[2] = strcpy(tmp1, readbuf);
        gtk_clist_append(GTK_CLIST(users_clist), users_str_list);
    }
    pclose(pipein_fp);

    // --------------------------------
    // ---> Fill Entry: IP Address
    // --------------------------------
    sprintf(readbuf, " ");
    // Use shell to get local machine IP Address
    if ((pipein_fp =
        popen("netstat -i eth0 -e | grep -i Bcast | cut -d':' -f2 | cut -d' ' -f1 | tr -d '\n'", "r")) == NULL) {
        perror("popen"); exit(1);
    }
    fgets(readbuf, 80, pipein_fp);
    pclose(pipein_fp);
    strcpy(IPaddress, readbuf);
    gtk_entry_set_text(GTK_ENTRY(entry_ipaddr), (gchar *)IPaddress);

    // `ifconfig` didnt work (RH 7.2), so switching to `netstat` for 0.9.8 and on
    // GNU C TCP/IP API's still not "doing it" for me (01/2002)... :-/
/*
    if (strlen(IPaddress) <= 0) {
    hostentAnswer = gethostent();
    printf("DEBUG: hostentAnswer = %s \n", hostentAnswer->h_name);
    printf("DEBUG: in_addr = %ld \n", (long int)hostentAnswer->h_addr);
    printf("DEBUG: inet_addr = %d \n", hostentAnswer->h_addr_list[1]);
//    printf("DEBUG: in_addr = %ld \n", netAddr->s_addr);
//    printf("DEBUG: inet_nsap_ntoa = %s \n", *inet_nsap_ntoa);
        gethostname(hostname, sizeof(hostname));
        hostentAnswer = gethostbyname(hostname);
        if ((h_errno != TRY_AGAIN) && (h_errno != NO_RECOVERY) && (h_errno != NO_ADDRESS)) {
            strcpy(IPaddress, inet_ntoa(*(struct in_addr*)hostentAnswer->h_addr));
        }
        else {
            strcpy(IPaddress, "Cant Determine IPADDR!");
        }
    }
*/

    // --------------------------------
    // ---> Fill Entries: PROCS- sleeping, running, zombie (parse /proc)
    // --------------------------------
    dir = opendir("/proc/");
    while ((dinfo = readdir(dir)) != 0) {

        // Step through /proc directory name one letter at-a-time
        dir_chr = dinfo->d_name;
        while (*dir_chr) {
            if ((*dir_chr >= '0') && (*dir_chr <= '9')) {
                dir_chr++;
                //printf("DEBUG: dir_chr = %s \n", dir_chr);
            }
            else {
                break;
            }
        }

        if (*dir_chr) {
            continue;
        }

        // Read each entry in the /proc directory
        strcpy(proc_file_name, "/proc/");
        strcat(proc_file_name, dinfo->d_name);
        strcat(proc_file_name, "/stat");

        pipein_fp = fopen(proc_file_name, "r");
        fscanf(pipein_fp, "%*d %s %c", proc_buffer, &proc_state);
        fclose(pipein_fp);

        // Keep running total of procs states
        if (proc_state == 'S') {
            num_proc_sleep++;
        }
        else if (proc_state == 'R') {
            num_proc_run++;
        }
        else if (proc_state == 'Z') {
            num_proc_zombie++;
        }
        else if (proc_state == 'T') {
            num_proc_stopped++;
        }
        else {
            num_proc_sleep++;
        }
        num_proc_total++;
    }

    // Set Process Summary ProgressBars
    gtk_progress_configure((GtkProgress *)progressbar_sleep,
                           (gfloat)num_proc_sleep,  0.0, (gfloat)num_proc_total);
    gtk_progress_configure((GtkProgress *)progressbar_run,
                           (gfloat)num_proc_run,    0.0, (gfloat)num_proc_total);
    gtk_progress_configure((GtkProgress *)progressbar_zombie,
                           (gfloat)num_proc_zombie, 0.0, (gfloat)num_proc_total);
    gtk_progress_configure((GtkProgress *)progressbar_stopped,
                           (gfloat)num_proc_stopped, 0.0, (gfloat)num_proc_total);


    // --------------------------------
    // ---> Fill Entries: Hardware & Kernel
    // --------------------------------
    // Get hardware (cpu) string
    // Macintosh & Intel machines have different /proc/cpuinfo files
    sprintf(readbuf, " ");
    if ((pipein_fp =
        popen("cat /proc/cpuinfo | grep -i pmac | tr -d '\n'", "r")) == NULL) {
        perror("popen"); exit(1);
    }
    fgets(readbuf, 80, pipein_fp);
    pclose(pipein_fp);

    // Motorola (PowerPC) Motherboard
    if (strlen(readbuf) > 1) {
        if ((pipein_fp =
            popen("cat /proc/cpuinfo | grep -i machine | sed -e 's/ //' | sed -e 's/Macintosh/Mac/i' | tr -d '\n'", "r")) == NULL) {
            perror("popen"); exit(1);
        }
        fgets(readbuf, 80, pipein_fp);
        hardware_str_ptr = strtok(readbuf, ":");
        hardware_machine = strtok(NULL, ":");
        pclose(pipein_fp);
        strcpy(hardware_str, hardware_machine);
        strcat(hardware_str, " ");

        if ((pipein_fp =
            popen("cat /proc/cpuinfo | grep -i cpu | sed -e 's/ //' | tr -d '\n'", "r")) == NULL) {
            perror("popen"); exit(1);
        }
        fgets(readbuf, 80, pipein_fp);
        hardware_str_ptr = strtok(readbuf, ":");
        hardware_cpu = strtok(NULL, ":");
        pclose(pipein_fp);
        strcat(hardware_str, hardware_cpu);
        strcat(hardware_str, " ");

        if ((pipein_fp =
            popen("cat /proc/cpuinfo | grep -i clock | sed -e 's/ //' | tr -d '\n'", "r")) == NULL) {
            perror("popen"); exit(1);
        }
        fgets(readbuf, 80, pipein_fp);
        hardware_str_ptr = strtok(readbuf, ":");
        hardware_clock = strtok(NULL, ":");
        pclose(pipein_fp);
        strcat(hardware_str, hardware_clock);
    }
    // Intel Motherboard (i hope!)
    else {
        sprintf(readbuf, " "); // Clear string before next use
        if ((pipein_fp =
            popen("cat /proc/cpuinfo | grep -i 'model name' | cut -d':' -f2 | sed -e 's/ //' | tr -d '\n'", "r")) == NULL) {
            perror("popen"); exit(1);
        }
        fgets(readbuf, 80, pipein_fp);
        pclose(pipein_fp);

        if (strlen(readbuf) > 1) {
            hardware_machine = readbuf;
        }
        else {
            hardware_machine = "Unknown";
        }
        strcpy(hardware_str, hardware_machine);
        strcat(hardware_str, " ");

        sprintf(readbuf, " "); // Clear string before next use
        if ((pipein_fp =
            popen("cat /proc/cpuinfo | grep -i 'cpu mhz' | sed -e 's/ //g' | tr -d '\n'", "r")) == NULL) {
            perror("popen"); exit(1);
        }
        fgets(readbuf, 80, pipein_fp);
        pclose(pipein_fp);
        if (strlen(readbuf) > 1) {
            hardware_str_ptr = strtok(readbuf, ":");
            hardware_cpu = strtok(NULL, ":");
        }
        else {
            hardware_cpu = "Unknown";
        }
        strcat(hardware_str, hardware_cpu);
    }

    // Get version string
    if ((pipein_fp = popen("cat /proc/version", "r")) == NULL) {
        perror("popen"); exit(1);
    }
    fgets(readbuf, 80, pipein_fp);
    kernel_str_ptr = strtok(readbuf, " ");
    kernel_str_ptr = strtok(NULL, " ");
    kernel_str_ptr = strtok(NULL, " ");
    pclose(pipein_fp);

    // Compose final kernel string
//    strcpy(kernel_str, "Version ");
    strcat(kernel_str, kernel_str_ptr);

//    gtk_entry_set_text(GTK_ENTRY(entry_hardware), hardware_str);
//    gtk_entry_set_text(GTK_ENTRY(entry_kernel),   kernel_str);

//    brent - 04/28/01 - perhaps we should ditch the texboxes and use LABELS (?)
//          - moving to 0.9.5
    gtk_label_set_justify(GTK_LABEL(label_hardware), GTK_JUSTIFY_RIGHT);
    gtk_label_set_text(GTK_LABEL(label_hardware), hardware_str);
    gtk_label_set_text(GTK_LABEL(label_kernel),   kernel_str);

}


void
on_users_clist_click_column (GtkCList *clist,
                             gint      column,
                             gpointer  user_data)
{
    // Get pointers to window widgets, declare local vars
    GtkWidget *users_clist = lookup_widget(app1, "users_clist");
    static int current_col = -1;

    // Set the column number to apply sort to
    gtk_clist_set_sort_column(GTK_CLIST(users_clist), column);

    // Choose sort type to pass to GTK (NULL = default, which is good for CHAR strings)
    gtk_clist_set_compare_func(GTK_CLIST(users_clist), NULL);

    // Based upon current sort (ASC or DESC), choose sort order for this iteration.
    if (current_col == column) {
        gtk_clist_set_sort_type(GTK_CLIST(users_clist), GTK_SORT_DESCENDING);
        current_col = -1;
    }
    else {
        gtk_clist_set_sort_type(GTK_CLIST(users_clist), GTK_SORT_ASCENDING);
        current_col = column;
    }

    // Sort the column in users_clist widget
    gtk_clist_sort(GTK_CLIST(users_clist));
}


// ------------------------------------------------------------------------- //
// -------------------->      Printing Function     <----------------------- //
// ------------------------------------------------------------------------- //

void
on_button_print_okay_clicked (GtkButton *button, gpointer user_data)
{
    // Get pointers to window widgets and declare local variables.
    GtkWidget *radiobutton_lpr   = lookup_widget(dialog_print, "radiobutton_lpr");
    GtkWidget *radiobutton_other = lookup_widget(dialog_print, "radiobutton_other");
    GtkWidget *entry_command     = lookup_widget(dialog_print, "entry_command");
    gchar     *entry_text        = "";
    char       print_command[80] = "";

    // Determine which print command to use: 'lpr' or "users choice"
    if (GTK_TOGGLE_BUTTON(radiobutton_lpr)->active) {
        strcpy(print_command, "lpr");
    }
    else if (GTK_TOGGLE_BUTTON(radiobutton_other)->active) {
        entry_text = gtk_entry_get_text((GtkEntry *) entry_command);
        sprintf(print_command, "%s", entry_text);

        if (strlen(print_command) == 0) {
            // Create Standard GNOME Error Dialog on-the-fly (i love these!)
            gnome_dialog_run_and_close(GNOME_DIALOG(gnome_error_dialog
                ("You must enter an alternate Print Command since you chose that option.")));
            return;
        }
    }

    // Pipe print command and voila!, the doc is printed.
    strcat(print_command, " /tmp/gwcc_out.txt");
    if (system(print_command) == -1) {
//        sprintf(entry_text, "The System Call '%s' failed!", print_command);
        gnome_dialog_run_and_close(GNOME_DIALOG(gnome_error_dialog(entry_text)));
    }

    // Destroy Print Dialog
    gtk_widget_destroy(dialog_print);
}


void
on_button_print_cancel_clicked (GtkButton *button, gpointer user_data)
{
    // Cancel was clicked, so Destroy Dialog Box.
    gtk_widget_destroy(dialog_print);
}

