#include "errno.h"
/* gwcc/src/prefs.c
 *
 * Copyright (C) 2000-2001 by Brent D. Ely
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

// Good old config.h
   #ifdef HAVE_CONFIG_H
   #  include <config.h>
   #endif

// Include Libraries
   #include <gnome.h>              /* (what do u think?) */
   #include <string.h>             /* for string functs  */
   #include <fcntl.h>              /* for open() command */
   #include <sys/stat.h>           /* for mkdir() funct  */
   #include "prefs.h"
   #include "utils.h"
   #include "interface.h"
   #include "support.h"

// Global Variables
   // Declare refs to app1 and others so we can interact with our widgets!
   extern GtkWidget *app1;          // The Main Application Window
   extern GtkWidget *propertybox1;  // Dialog - Preferences
   GtkWidget *dialog_save_as;       // Dialog - "Save As..."
   GtkWidget *dialog_print;         // Dialog - "Print"

   // Global Preference Variables
   // (Tabs: General, Workstation Utils, Grep, Network).
   int    file_save_flag       = 0;
   int    timestamp_saved_file = 0;
   gchar *fixed_font_ptr;
   char   fixed_font[240]      = "";
   int    display_units        = 0;
   int    timestamp_scn_output = 0;
   int    toolbar_setting_int  = 0;

   int  ifconfig_flag        = 0;
   int  df_flag_a            = 0;
   int  df_flag_h            = 0;
   int  df_flag_t            = 0;
   int  netstat_flag         = 0;

   int  grep_flag            = 0;
   int  use_grep_i_flag      = 0;
   int  omit_grep_sh_output  = 0;

   int  ping_c               = 0;
   int  ping_w               = 0;
   int  ping_i               = 0;
   int  use_traceroute_n     = 0;
   int  use_host_v           = 0;
   int  finger_flag          = 0;
   int  nslookup_retries     = 0;
   int  nslookup_timeout     = 0;


// ------------------------------------------------------------------------- //
// ----------------->     Read Preferences From Disk    <------------------- //
// ------------------------------------------------------------------------- //

void
read_prefs_from_disk()
{
    // Local Variables
    char  *name           = "";
    char  *env_homedir    = "";
    char   file_name[80]  = "";
    char   dir_name[80]   = "";
    char   in_string[80]  = "";
    char   dialog_str[80] = "";
    int    value          = 0;
    FILE  *file_ptr;
    extern int errno; // Comes from <sys/stat.h> - used for determining exact error. EEXIST

    // Determine users home directory
    env_homedir = getenv("HOME");
    if (env_homedir == NULL) {
        gnome_dialog_run_and_close(GNOME_DIALOG(gnome_error_dialog
            ("Cannot determine your $HOME directory... Thats weird.")));
        return;
    }

    // Construct final file_ptr string to prefs file
    strcpy(file_name, env_homedir);
    strcat(file_name, "/.gwcc/prefs.conf");

    // Check for existence of .gwcc sub-directory / Create one if necc.
    // NOTE: This also covers the SAVE PREFS function below - because this is always called first!
    strcpy(dir_name, env_homedir);
    strcat(dir_name, "/.gwcc");
    if ((mkdir(dir_name, (S_IRWXU|S_IRGRP|S_IXGRP|S_IROTH|S_IXOTH)) != 0) && (errno != 17)) {
//        printf("DEBUG: errno = %d \n", errno);
        sprintf(dialog_str, "Could not create PREFS directory (%s) - please do so yourself.", dir_name);
        gnome_dialog_run_and_close(GNOME_DIALOG(gnome_error_dialog(dialog_str)));
        return;
    }

    // Read Prefs File one line at a time -and- Assign Variables as we go
    if ((file_ptr = fopen(file_name, "r")) == NULL) {
        // Maybe this is the first time the program was run.. just ignore this exception.
        return;
    }
    while (fgets(in_string, 80, file_ptr) != NULL) {
        name  = strtok(in_string, "=");

        // Assign Value to fixed_font Variable (its a char, so its different)
        if ((strcmp(name, "fixed_font") == 0) && (strlen(name) <= 10)) {
            sprintf(fixed_font, "%s", strtok(NULL, "="));
        }
        else {
            value = atoi(strtok(NULL, "="));
        }

        // Assign Values to Variables
        if (strcmp(name, "file_save_flag")       == 0) file_save_flag       = value;
        if (strcmp(name, "timestamp_saved_file") == 0) timestamp_saved_file = value;
        if (strcmp(name, "display_units")        == 0) display_units        = value;
        if (strcmp(name, "timestamp_scn_output") == 0) timestamp_scn_output = value;
        if (strcmp(name, "toolbar_setting_int")  == 0) toolbar_setting_int  = value;

        if (strcmp(name, "ifconfig_flag")        == 0) ifconfig_flag        = value;
        if (strcmp(name, "df_flag_a")            == 0) df_flag_a            = value;
        if (strcmp(name, "df_flag_h")            == 0) df_flag_h            = value;
        if (strcmp(name, "df_flag_t")            == 0) df_flag_t            = value;
        if (strcmp(name, "netstat_flag")         == 0) netstat_flag         = value;

        if (strcmp(name, "grep_flag")            == 0) grep_flag            = value;
        if (strcmp(name, "use_grep_i_flag")      == 0) use_grep_i_flag      = value;
        if (strcmp(name, "omit_grep_sh_output")  == 0) omit_grep_sh_output  = value;

        if (strcmp(name, "ping_c")               == 0) ping_c               = value;
        if (strcmp(name, "ping_w")               == 0) ping_w               = value;
        if (strcmp(name, "ping_i")               == 0) ping_i               = value;
        if (strcmp(name, "use_traceroute_n")     == 0) use_traceroute_n     = value;
        if (strcmp(name, "use_host_v")           == 0) use_host_v           = value;
        if (strcmp(name, "finger_flag")          == 0) finger_flag          = value;
        if (strcmp(name, "nslookup_retries")     == 0) nslookup_retries     = value;
        if (strcmp(name, "nslookup_timeout")     == 0) nslookup_timeout     = value;
    }
    fclose(file_ptr);
}


// ------------------------------------------------------------------------- //
// ------------------>      Save Preferences To Disk    <------------------- //
// ------------------------------------------------------------------------- //

int
save_all_changes (GnomePropertyBox *gnomepropertybox,
                  gint              arg1,
                  gpointer          user_data)
{
    // Local Variables
    char  *env_homedir    = "";
    char   file_name[80]  = "";
    char   out_string[80] = "";
    FILE  *file_ptr;

    // Determine users home directory
    env_homedir = getenv("HOME");
    if (env_homedir == NULL) {
        printf("ERROR: Cannot determine HOME directory!");
        return(-1);
    }

    // Construct final file_ptr string to prefs file
    strcpy(file_name, env_homedir);
    strcat(file_name, "/.gwcc/prefs.conf");

    // Write Prefs File to user's home directory under gwcc subdir
    if ((file_ptr = fopen(file_name, "w")) != NULL) {
        sprintf(out_string, "file_save_flag=%d\n", file_save_flag);
        fputs(out_string, file_ptr);
        sprintf(out_string, "timestamp_saved_file=%d\n", timestamp_saved_file);
        fputs(out_string, file_ptr);
        sprintf(out_string, "fixed_font=%s=\n", fixed_font);
        fputs(out_string, file_ptr);
        sprintf(out_string, "display_units=%d\n", display_units);
        fputs(out_string, file_ptr);
        sprintf(out_string, "timestamp_scn_output=%d\n", timestamp_scn_output);
        fputs(out_string, file_ptr);
        sprintf(out_string, "toolbar_setting_int=%d\n", toolbar_setting_int);
        fputs(out_string, file_ptr);

        sprintf(out_string, "ifconfig_flag=%d\n", ifconfig_flag);
        fputs(out_string, file_ptr);
        sprintf(out_string, "df_flag_a=%d\n", df_flag_a);
        fputs(out_string, file_ptr);
        sprintf(out_string, "df_flag_h=%d\n", df_flag_h);
        fputs(out_string, file_ptr);
        sprintf(out_string, "df_flag_t=%d\n", df_flag_t);
        fputs(out_string, file_ptr);
        sprintf(out_string, "netstat_flag=%d\n", netstat_flag);
        fputs(out_string, file_ptr);

        sprintf(out_string, "grep_flag=%d\n", grep_flag);
        fputs(out_string, file_ptr);
        sprintf(out_string, "use_grep_i_flag=%d\n", use_grep_i_flag);
        fputs(out_string, file_ptr);
        sprintf(out_string, "omit_grep_sh_output=%d\n", omit_grep_sh_output);
        fputs(out_string, file_ptr);

        sprintf(out_string, "ping_c=%d\n", ping_c);
        fputs(out_string, file_ptr);
        sprintf(out_string, "ping_w=%d\n", ping_w);
        fputs(out_string, file_ptr);
        sprintf(out_string, "ping_i=%d\n", ping_i);
        fputs(out_string, file_ptr);
        sprintf(out_string, "use_traceroute_n=%d\n", use_traceroute_n);
        fputs(out_string, file_ptr);
        sprintf(out_string, "use_host_v=%d\n", use_host_v);
        fputs(out_string, file_ptr);
        sprintf(out_string, "finger_flag=%d\n", finger_flag);
        fputs(out_string, file_ptr);
        sprintf(out_string, "nslookup_retries=%d\n", nslookup_retries);
        fputs(out_string, file_ptr);
        sprintf(out_string, "nslookup_timeout=%d\n", nslookup_timeout);
        fputs(out_string, file_ptr);
    }
    fclose(file_ptr);

    return(0);
}


// ------------------------------------------------------------------------- //
// ----->     Fill Preferences Dialog with Values from Prefs File    <------ //
// ------------------------------------------------------------------------- //

void
fill_prefs_dialog()
{
    // Get pointers to window widgets and Declare local variables
    GtkWidget * checkbutton_ts_files       = lookup_widget(propertybox1, "checkbutton_ts_files");
    GtkWidget * radiobutton_save_overwrite = lookup_widget(propertybox1, "radiobutton_save_overwrite");
    GtkWidget * radiobutton_save_append    = lookup_widget(propertybox1, "radiobutton_save_append");
    GtkWidget * radiobutton_save_seq       = lookup_widget(propertybox1, "radiobutton_save_seq");
    GtkWidget * check_add_timestamp        = lookup_widget(propertybox1, "check_add_timestamp");
    GtkWidget * fontpicker1                = lookup_widget(propertybox1, "fontpicker1");
    GtkWidget * radiobutton_units_m        = lookup_widget(propertybox1, "radiobutton_units_m");
    GtkWidget * radiobutton_units_k        = lookup_widget(propertybox1, "radiobutton_units_k");
    GtkWidget * optionmenu_toolbar         = lookup_widget(propertybox1, "optionmenu_toolbar");

    GtkWidget * ifconfig_arp               = lookup_widget(propertybox1, "ifconfig_arp");
    GtkWidget * ifconfig_promisc           = lookup_widget(propertybox1, "ifconfig_promisc");
    GtkWidget * ifconfig_allmulti          = lookup_widget(propertybox1, "ifconfig_allmulti");
    GtkWidget * check_df_a                 = lookup_widget(propertybox1, "check_df_a");
    GtkWidget * check_df_h                 = lookup_widget(propertybox1, "check_df_h");
    GtkWidget * check_df_T                 = lookup_widget(propertybox1, "check_df_T");
    GtkWidget * netstat_u                  = lookup_widget(propertybox1, "netstat_u");
    GtkWidget * netstat_M                  = lookup_widget(propertybox1, "netstat_M");
    GtkWidget * netstat_i                  = lookup_widget(propertybox1, "netstat_i");
    GtkWidget * netstat_p                  = lookup_widget(propertybox1, "netstat_p");

    GtkWidget * ps_flag_none               = lookup_widget(propertybox1, "ps_flag_none");
    GtkWidget * ps_flag_e                  = lookup_widget(propertybox1, "ps_flag_e");
    GtkWidget * ps_flag_eh                 = lookup_widget(propertybox1, "ps_flag_eh");
    GtkWidget * ps_flag_efh                = lookup_widget(propertybox1, "ps_flag_efh");
    GtkWidget * checkbutton_grep_case      = lookup_widget(propertybox1, "checkbutton_grep_case");
    GtkWidget * checkbutton_omit_sh        = lookup_widget(propertybox1, "checkbutton_omit_sh");

    GtkWidget * spinbutton1                = lookup_widget(propertybox1, "spinbutton1");
    GtkWidget * spinbutton2                = lookup_widget(propertybox1, "spinbutton2");
    GtkWidget * spinbutton3                = lookup_widget(propertybox1, "spinbutton3");
    GtkWidget * checkbutton_traceroute_n   = lookup_widget(propertybox1, "checkbutton_traceroute_n");
    GtkWidget * checkbutton_host_v         = lookup_widget(propertybox1, "checkbutton_host_v");
    GtkWidget * finger_s                   = lookup_widget(propertybox1, "finger_s");
    GtkWidget * finger_l                   = lookup_widget(propertybox1, "finger_l");
    GtkWidget * spinbutton4                = lookup_widget(propertybox1, "spinbutton4");
    GtkWidget * spinbutton5                = lookup_widget(propertybox1, "spinbutton5");
    GtkWidget * menu;

//    GtkWidget * apply                      = lookup_widget(propertybox1, "apply");

    // NOTE: This function causes the "Apply" button to become active, we have to..
    // Disconnect "Apply" button signal handler or else "Apply" comes on right away!
//    gtk_widget_set_sensitive((GtkWidget *) apply,  FALSE);
//    Brent: 05/31/01 - i have no idea what the hell this widget is identified by.. or what else to do???


    // -------------------------------------
    // General Prefs Tab: File Save Options
    // -------------------------------------
    if (file_save_flag == 0) {
        gtk_toggle_button_set_active((GtkToggleButton *) radiobutton_save_overwrite, 1);
    }
    else if (file_save_flag == 1) {
        gtk_toggle_button_set_active((GtkToggleButton *) radiobutton_save_append, 1);
    }
    else if (file_save_flag == 2) {
        gtk_toggle_button_set_active((GtkToggleButton *) radiobutton_save_seq, 1);
    }

    // General Prefs Tab: Fixed Font
    // -------------------------------------
    if (strlen(fixed_font) >= 1) {
        gnome_font_picker_set_font_name((GnomeFontPicker *) fontpicker1, (gchar *)fixed_font);
    }

    // General Prefs Tab: Units Options
    // -------------------------------------
    if (display_units == 0) {
        gtk_toggle_button_set_active((GtkToggleButton *) radiobutton_units_m, 1);
    }
    else if (display_units == 1) {
        gtk_toggle_button_set_active((GtkToggleButton *) radiobutton_units_k, 1);
    }

    // General Prefs Tab: Timestamp Options
    // -------------------------------------
    gtk_toggle_button_set_active((GtkToggleButton *) checkbutton_ts_files, timestamp_saved_file);
    gtk_toggle_button_set_active((GtkToggleButton *) check_add_timestamp,  timestamp_scn_output);

    // General Prefs Tab: Toolbar Setting
    // -------------------------------------
    // Two Tasks are *required* here:
    //   1) set active menu item index - gives the menu widget desired index value
    //   2) set OptionMenu history index - shows the selected menu item on screen
    // If you leave out step 2 the value is correct, but it wont show on screen!!
    menu = GTK_OPTION_MENU(optionmenu_toolbar)->menu;
    gtk_menu_set_active(GTK_MENU(menu), toolbar_setting_int);
    gtk_option_menu_set_history(GTK_OPTION_MENU(optionmenu_toolbar), toolbar_setting_int);


    // -------------------------------------
    // Workstation Utils Tab:
    // -------------------------------------
    if (ifconfig_flag == 0) {
        gtk_toggle_button_set_active((GtkToggleButton *) ifconfig_arp, 1);
    }
    else if (ifconfig_flag == 1) {
        gtk_toggle_button_set_active((GtkToggleButton *) ifconfig_promisc, 1);
    }
    else if (ifconfig_flag == 2) {
        gtk_toggle_button_set_active((GtkToggleButton *) ifconfig_allmulti, 1);
    }

    // Workstation Utils Tab: df options
    // -------------------------------------
    gtk_toggle_button_set_active((GtkToggleButton *) check_df_a, df_flag_a);
    gtk_toggle_button_set_active((GtkToggleButton *) check_df_h, df_flag_h);
    gtk_toggle_button_set_active((GtkToggleButton *) check_df_T, df_flag_t);

    // Workstation Utils Tab: netstat options
    // -------------------------------------
    if (netstat_flag == 0) {
        gtk_toggle_button_set_active((GtkToggleButton *) netstat_u, 1);
    }
    else if (netstat_flag == 1) {
        gtk_toggle_button_set_active((GtkToggleButton *) netstat_M, 1);
    }
    else if (netstat_flag == 2) {
        gtk_toggle_button_set_active((GtkToggleButton *) netstat_i, 1);
    }
    else if (netstat_flag == 3) {
        gtk_toggle_button_set_active((GtkToggleButton *) netstat_p, 1);
    }


    // -------------------------------------
    // Grep Prefs Tab:
    // -------------------------------------
    if (grep_flag == 0) {
        gtk_toggle_button_set_active((GtkToggleButton *) ps_flag_none, 1);
    }
    else if (grep_flag == 1) {
        gtk_toggle_button_set_active((GtkToggleButton *) ps_flag_e, 1);
    }
    else if (grep_flag == 2) {
        gtk_toggle_button_set_active((GtkToggleButton *) ps_flag_eh, 1);
    }
    else if (grep_flag == 3) {
        gtk_toggle_button_set_active((GtkToggleButton *) ps_flag_efh, 1);
    }
    gtk_toggle_button_set_active((GtkToggleButton *) checkbutton_grep_case, use_grep_i_flag);
    gtk_toggle_button_set_active((GtkToggleButton *) checkbutton_omit_sh, omit_grep_sh_output);


    // -------------------------------------
    // Network Prefs Tab: Ping Options
    // -------------------------------------
    gtk_spin_button_set_value((GtkSpinButton *) spinbutton1, (float) ping_c);
    gtk_spin_button_set_value((GtkSpinButton *) spinbutton2, (float) ping_w);
    gtk_spin_button_set_value((GtkSpinButton *) spinbutton3, (float) ping_i);

    // Network Prefs Tab: Traceroute Options
    // -------------------------------------
    gtk_toggle_button_set_active((GtkToggleButton *) checkbutton_traceroute_n,  use_traceroute_n);

    // Network Prefs Tab: Host Options
    // -------------------------------------
    gtk_toggle_button_set_active((GtkToggleButton *) checkbutton_host_v,  use_host_v);

    // Network Prefs Tab: Finger Options
    // -------------------------------------
    if (finger_flag == 0) {
        gtk_toggle_button_set_active((GtkToggleButton *) finger_s, 1);
    }
    else if (finger_flag == 1) {
        gtk_toggle_button_set_active((GtkToggleButton *) finger_l, 1);
    }

    // Network Prefs Tab: Nslookup Options
    // -------------------------------------
    gtk_spin_button_set_value((GtkSpinButton *) spinbutton4, (float) nslookup_retries);
    gtk_spin_button_set_value((GtkSpinButton *) spinbutton5, (float) nslookup_timeout);

    return;
}


// ------------------------------------------------------------------------- //
// ------------------------------------------------------------------------- //
// ---------------->      Preferences Dialog Callbacks    <----------------- //
// ------------------------------------------------------------------------- //
// ------------------------------------------------------------------------- //

void
on_propertybox1_apply (GnomePropertyBox *gnomepropertybox,
                       gint              arg1,
                       gpointer          user_data)
{
    // This function is called whenever the <APPLY> button is clicked, --OR--
    // when <OK> is clicked after making a change to a widget and
    // gnome_property_box_changed() has been called.

    // Gnome docs had this (?)
    if (arg1 != -1) {
        return;
    }

    // PREF: Toolbar Setting
    // As of 12/2001, GLADE doe snot support callbacks for GtkOptionMenu (see the FAQ)
    // Brents Cheap Way Out(tm):
    // just get the setting when the dialog is closing and before pref values
    // are written to disk -> we dont really need any functionality outside of this.
    on_optionmenu_toolbar_clicked(NULL, NULL);
    
    // Call function to save all prefs variables to gwcc.conf file!
    if (save_all_changes(gnomepropertybox, 0, NULL) != 0) {
        gnome_dialog_run_and_close(GNOME_DIALOG(gnome_error_dialog
            ("OOPS! Something Went Wrong!\nPlease email me the Error Dialog string...")));
    }

    // Apply Toolbar Settings to Main Application Window (utils.c)
    // as soon as we can (on Pref Dialog <OK> or <Apply> button pressed)
    set_current_toolbar_style(NULL);
}


// ------------------------------------------------------------------------- //
// --------------->    General Options TAB -  Callbacks    <---------------- //
// ------------------------------------------------------------------------- //

// Save File Options: Timestamp Files?
// -----------------------------------
void
on_checkbutton_ts_files_toggled (GtkToggleButton *togglebutton, gpointer user_data)
{
    // Get pointers to window widgets and declare local variables
    GtkWidget * checkbutton_ts_files = lookup_widget(propertybox1, "checkbutton_ts_files");

    // Activate 'Apply' button and 'Save' callback in Prefs Dialog.
    gnome_property_box_changed((GnomePropertyBox*)propertybox1);

    // All Global Variables are set here (not in individual callbacks).
    if (GTK_TOGGLE_BUTTON(checkbutton_ts_files)->active) {
        timestamp_saved_file = 1;
    }
    else {
        timestamp_saved_file = 0;
    }
}


// Save File Options: OVERWRITE
// ----------------------------
void
on_radiobutton_save_overwrite_toggled (GtkToggleButton *button, gpointer user_data)
{
    // Get pointers to window widgets and declare local variables
    GtkWidget * radiobutton_save_overwrite = lookup_widget(propertybox1, "radiobutton_save_overwrite");

    // Activate 'Apply' button and 'Save' callback in Prefs Dialog.
    gnome_property_box_changed((GnomePropertyBox *) propertybox1);

    // Set Preferences Value
    if (GTK_TOGGLE_BUTTON(radiobutton_save_overwrite)->active) {
        file_save_flag = 0;
    }
}


// Save File Options: APPEND
// -------------------------
void
on_radiobutton_save_append_toggled (GtkToggleButton *button, gpointer user_data)
{
    // Get pointers to window widgets and declare local variables
    GtkWidget * radiobutton_save_append = lookup_widget(propertybox1, "radiobutton_save_append");

    // Activate 'Apply' button and 'Save' callback in Prefs Dialog.
    gnome_property_box_changed((GnomePropertyBox *) propertybox1);

    // Set Preferences Value
    if (GTK_TOGGLE_BUTTON(radiobutton_save_append)->active) {
        file_save_flag = 1;
    }
}


// Save File Options: SEQUENTIAL
// -----------------------------
void
on_radiobutton_save_seq_toggled (GtkToggleButton *button, gpointer user_data)
{
    // Get pointers to window widgets and declare local variables
    GtkWidget * radiobutton_save_seq = lookup_widget(propertybox1, "radiobutton_save_seq");

    // Activate 'Apply' button and 'Save' callback in Prefs Dialog.
    gnome_property_box_changed((GnomePropertyBox *) propertybox1);

    // Set Preferences Value
    if (GTK_TOGGLE_BUTTON(radiobutton_save_seq)->active) {
        file_save_flag = 2;
    }
}


// General Options: Fixed Font Size
// ---------------------------------
void
on_fontpicker1_font_set (GnomeFontPicker *gnomefontpicker,
                         gchar           *arg1,
                         gpointer         user_data)
{
    // Get pointers to window widgets and declare local variables
    GtkWidget * fontpicker1 = lookup_widget(propertybox1, "fontpicker1");

    fixed_font_ptr = gnome_font_picker_get_font_name((GnomeFontPicker*) fontpicker1);
    sprintf(fixed_font, "%s", fixed_font_ptr);

    // Activate 'Apply' button and 'Save' callback in Prefs Dialog.
    gnome_property_box_changed((GnomePropertyBox *) propertybox1);
}


void
on_radiobutton_units_m_toggled (GtkToggleButton *togglebutton, gpointer user_data)
{
    // Get pointers to window widgets and declare local variables
    GtkWidget * radiobutton_units_m = lookup_widget(propertybox1, "radiobutton_units_m");

    // Activate 'Apply' button and 'Save' callback in Prefs Dialog.
    gnome_property_box_changed((GnomePropertyBox *) propertybox1);

    // Set Preferences Value
    if (GTK_TOGGLE_BUTTON(radiobutton_units_m)->active) {
        display_units = 0;
    }
}


void
on_radiobutton_units_k_toggled  (GtkToggleButton *togglebutton, gpointer user_data)
{
    // Get pointers to window widgets and declare local variables
    GtkWidget * radiobutton_units_k = lookup_widget(propertybox1, "radiobutton_units_k");

    // Activate 'Apply' button and 'Save' callback in Prefs Dialog.
    gnome_property_box_changed((GnomePropertyBox *) propertybox1);

    // Set Preferences Value
    if (GTK_TOGGLE_BUTTON(radiobutton_units_k)->active) {
        display_units = 1;
    }
}


// General Options: Timestamp Output
// ---------------------------------
void
on_check_add_timestamp_toggled (GtkToggleButton *togglebutton, gpointer user_data)
{
    // Get pointers to window widgets and declare local variables
    GtkWidget * check_add_timestamp = lookup_widget(propertybox1, "check_add_timestamp");

    // Activate 'Apply' button and 'Save' callback in Prefs Dialog.
    gnome_property_box_changed((GnomePropertyBox *) propertybox1);

    // Set Preferences Value
    if (GTK_TOGGLE_BUTTON(check_add_timestamp)->active) {
        timestamp_scn_output = 1;
    }
    else {
        timestamp_scn_output = 0;
    }
}


// ------------------------------------------------------------------------- //
// --------------->   Workstation Utils TAB -  Callbacks   <---------------- //
// ------------------------------------------------------------------------- //

void
on_ifconfig_arp_toggled  (GtkToggleButton *togglebutton, gpointer user_data)
{
    // Activate 'Apply' button and 'Save' callback in Prefs Dialog.
    gnome_property_box_changed((GnomePropertyBox*)propertybox1);

    // All Global Variables are set here (not in individual callbacks).
    ifconfig_flag = 0;
}


void
on_ifconfig_promisc_toggled  (GtkToggleButton *togglebutton, gpointer user_data)
{
    // Activate 'Apply' button and 'Save' callback in Prefs Dialog.
    gnome_property_box_changed((GnomePropertyBox*)propertybox1);

    // All Global Variables are set here (not in individual callbacks).
    ifconfig_flag = 1;
}


void
on_ifconfig_allmulti_toggled  (GtkToggleButton *togglebutton, gpointer user_data)
{
    // Activate 'Apply' button and 'Save' callback in Prefs Dialog.
    gnome_property_box_changed((GnomePropertyBox*)propertybox1);

    // All Global Variables are set here (not in individual callbacks).
    ifconfig_flag = 2;
}


void
on_check_df_a_toggled (GtkToggleButton *togglebutton, gpointer user_data)
{
    // Get pointers to window widgets and declare local variables
    GtkWidget * check_df_a = lookup_widget(propertybox1, "check_df_a");

    // Activate 'Apply' button and 'Save' callback in Prefs Dialog.
    gnome_property_box_changed((GnomePropertyBox *) propertybox1);

    // Set Preferences Value
    if (GTK_TOGGLE_BUTTON(check_df_a)->active) {
        df_flag_a = 1;
    }
    else {
        df_flag_a = 0;
    }
}


void
on_check_df_h_toggled (GtkToggleButton *togglebutton, gpointer user_data)
{
    // Get pointers to window widgets and declare local variables
    GtkWidget * check_df_h = lookup_widget(propertybox1, "check_df_h");

    // Activate 'Apply' button and 'Save' callback in Prefs Dialog.
    gnome_property_box_changed((GnomePropertyBox *) propertybox1);

    // Set Preferences Value
    if (GTK_TOGGLE_BUTTON(check_df_h)->active) {
        df_flag_h = 1;
    }
    else {
        df_flag_h = 0;
    }
}


void
on_check_df_T_toggled (GtkToggleButton *togglebutton, gpointer user_data)
{
    // Get pointers to window widgets and declare local variables
    GtkWidget * check_df_T = lookup_widget(propertybox1, "check_df_T");

    // Activate 'Apply' button and 'Save' callback in Prefs Dialog.
    gnome_property_box_changed((GnomePropertyBox *) propertybox1);

    // Set Preferences Value
    if (GTK_TOGGLE_BUTTON(check_df_T)->active) {
        df_flag_t = 1;
    }
    else {
        df_flag_t = 0;
    }
}


void
on_netstat_u_toggled  (GtkToggleButton *togglebutton, gpointer user_data)
{
    // Activate 'Apply' button and 'Save' callback in Prefs Dialog.
    gnome_property_box_changed((GnomePropertyBox*)propertybox1);

    // All Global Variables are set here (not in individual callbacks).
    netstat_flag = 0;
}


void
on_netstat_M_toggled  (GtkToggleButton *togglebutton, gpointer user_data)
{
    // Activate 'Apply' button and 'Save' callback in Prefs Dialog.
    gnome_property_box_changed((GnomePropertyBox*)propertybox1);

    // All Global Variables are set here (not in individual callbacks).
    netstat_flag = 1;
}


void
on_netstat_i_toggled  (GtkToggleButton *togglebutton, gpointer user_data)
{
    // Activate 'Apply' button and 'Save' callback in Prefs Dialog.
    gnome_property_box_changed((GnomePropertyBox*)propertybox1);

    // All Global Variables are set here (not in individual callbacks).
    netstat_flag = 2;
}


void
on_netstat_p_toggled  (GtkToggleButton *togglebutton, gpointer user_data)
{
    // Activate 'Apply' button and 'Save' callback in Prefs Dialog.
    gnome_property_box_changed((GnomePropertyBox*)propertybox1);

    // All Global Variables are set here (not in individual callbacks).
    netstat_flag = 3;
}


// ------------------------------------------------------------------------- //
// -------------->   Process Grep Utils TAB -  Callbacks   <---------------- //
// ------------------------------------------------------------------------- //

void
on_ps_flag_none_toggled  (GtkToggleButton *togglebutton, gpointer user_data)
{
    // Get pointers to window widgets and declare local variables
    GtkWidget * ps_flag_none = lookup_widget(propertybox1, "ps_flag_none");

    // Activate 'Apply' button and 'Save' callback in Prefs Dialog.
    gnome_property_box_changed((GnomePropertyBox *) propertybox1);

    // Set Preferences Value
    if (GTK_TOGGLE_BUTTON(ps_flag_none)->active) {
        grep_flag = 0;
    }
}


void
on_ps_flag_e_toggled  (GtkToggleButton *togglebutton, gpointer user_data)
{
    // Get pointers to window widgets and declare local variables
    GtkWidget * ps_flag_e = lookup_widget(propertybox1, "ps_flag_e");

    // Activate 'Apply' button and 'Save' callback in Prefs Dialog.
    gnome_property_box_changed((GnomePropertyBox *) propertybox1);

    // Set Preferences Value
    if (GTK_TOGGLE_BUTTON(ps_flag_e)->active) {
        grep_flag = 1;
    }
}


void
on_ps_flag_eh_toggled  (GtkToggleButton *togglebutton, gpointer user_data)
{
    // Get pointers to window widgets and declare local variables
    GtkWidget * ps_flag_eh = lookup_widget(propertybox1, "ps_flag_eh");

    // Activate 'Apply' button and 'Save' callback in Prefs Dialog.
    gnome_property_box_changed((GnomePropertyBox *) propertybox1);

    // Set Preferences Value
    if (GTK_TOGGLE_BUTTON(ps_flag_eh)->active) {
        grep_flag = 2;
    }
}


void
on_ps_flag_efh_toggled (GtkToggleButton *togglebutton, gpointer user_data)
{
    // Get pointers to window widgets and declare local variables
    GtkWidget * ps_flag_efh = lookup_widget(propertybox1, "ps_flag_efh");

    // Activate 'Apply' button and 'Save' callback in Prefs Dialog.
    gnome_property_box_changed((GnomePropertyBox *) propertybox1);

    // Set Preferences Value
    if (GTK_TOGGLE_BUTTON(ps_flag_efh)->active) {
        grep_flag = 3;
    }
}


void
on_checkbutton_grep_case_toggled       (GtkToggleButton *togglebutton,
                                        gpointer         user_data)
{
    // Get pointers to window widgets and declare local variables
    GtkWidget * checkbutton_grep_case = lookup_widget(propertybox1, "checkbutton_grep_case");

    // Activate 'Apply' button and 'Save' callback in Prefs Dialog.
    gnome_property_box_changed((GnomePropertyBox *) propertybox1);

    // Set Preferences Value
    if (GTK_TOGGLE_BUTTON(checkbutton_grep_case)->active) {
        use_grep_i_flag = 1;
    }
    else {
        use_grep_i_flag = 0;
    }
}


void
on_checkbutton_omit_sh_toggled  (GtkToggleButton *togglebutton, gpointer user_data)
{
    // Get pointers to window widgets and declare local variables
    GtkWidget * checkbutton_omit_sh = lookup_widget(propertybox1, "checkbutton_omit_sh");

    // Activate 'Apply' button and 'Save' callback in Prefs Dialog.
    gnome_property_box_changed((GnomePropertyBox *) propertybox1);

    // Set Preferences Value
    if (GTK_TOGGLE_BUTTON(checkbutton_omit_sh)->active) {
        omit_grep_sh_output = 1;
    }
    else {
        omit_grep_sh_output = 0;
    }
}


// ------------------------------------------------------------------------- //
// ---------------->      Network Options -  Callbacks    <----------------- //
// ------------------------------------------------------------------------- //

// PING Options: -c
void
on_spinbutton1_changed (GtkEditable *editable, gpointer user_data)
{
    // Get pointers to window widgets and declare local variables
    GtkWidget * spinbutton1 = lookup_widget(propertybox1, "spinbutton1");

    // Activate 'Apply' button and 'Save' callback in Prefs Dialog.
    gnome_property_box_changed((GnomePropertyBox *) propertybox1);

    // Set Preferences Value
    ping_c = gtk_spin_button_get_value_as_int((GtkSpinButton *) spinbutton1);
    //printf("DEBUG: ping_c = %d \n", ping_c);
}


// PING Options: -w
void
on_spinbutton2_changed (GtkEditable *editable, gpointer user_data)
{
    // Get pointers to window widgets and declare local variables
    GtkWidget * spinbutton2 = lookup_widget(propertybox1, "spinbutton2");

    // Activate 'Apply' button and 'Save' callback in Prefs Dialog.
    gnome_property_box_changed((GnomePropertyBox *) propertybox1);

    // Set Preferences Value
    ping_w = gtk_spin_button_get_value_as_int((GtkSpinButton *) spinbutton2);
}


// PING Options: -i
void
on_spinbutton3_changed (GtkEditable *editable, gpointer user_data)
{
    // Get pointers to window widgets and declare local variables
    GtkWidget * spinbutton3 = lookup_widget(propertybox1, "spinbutton3");

    // Activate 'Apply' button and 'Save' callback in Prefs Dialog.
    gnome_property_box_changed((GnomePropertyBox *) propertybox1);

    // Set Preferences Value
    ping_i = gtk_spin_button_get_value_as_int((GtkSpinButton *) spinbutton3);
}


void
on_checkbutton_traceroute_n_toggled (GtkToggleButton *togglebutton, gpointer user_data)
{
    // Get pointers to window widgets and declare local variables
    GtkWidget * checkbutton_traceroute_n = lookup_widget(propertybox1, "checkbutton_traceroute_n");

    // Activate 'Apply' button and 'Save' callback in Prefs Dialog.
    gnome_property_box_changed((GnomePropertyBox*)propertybox1);

    // All Global Variables are set here (not in individual callbacks).
    if (GTK_TOGGLE_BUTTON(checkbutton_traceroute_n)->active) {
        use_traceroute_n = 1;
    }
    else {
        use_traceroute_n = 0;
    }
}


void
on_checkbutton_host_v_toggled (GtkToggleButton *togglebutton, gpointer user_data)
{
    // Get pointers to window widgets and declare local variables
    GtkWidget * checkbutton_host_v = lookup_widget(propertybox1, "checkbutton_host_v");

    // Activate 'Apply' button and 'Save' callback in Prefs Dialog.
    gnome_property_box_changed((GnomePropertyBox*)propertybox1);

    // All Global Variables are set here (not in individual callbacks).
    if (GTK_TOGGLE_BUTTON(checkbutton_host_v)->active) {
        use_host_v = 1;
    }
    else {
        use_host_v = 0;
    }
}


void
on_finger_s_toggled (GtkToggleButton *togglebutton, gpointer user_data)
{
    // Activate 'Apply' button and 'Save' callback in Prefs Dialog.
    gnome_property_box_changed((GnomePropertyBox*)propertybox1);

    // All Global Variables are set here (not in individual callbacks).
    finger_flag = 0;
}


void
on_finger_l_toggled (GtkToggleButton *togglebutton, gpointer user_data)
{
    // Activate 'Apply' button and 'Save' callback in Prefs Dialog.
    gnome_property_box_changed((GnomePropertyBox*)propertybox1);

    // All Global Variables are set here (not in individual callbacks).
    finger_flag = 1;
}


void
on_spinbutton4_changed  (GtkEditable *editable, gpointer user_data)
{
    // Get pointers to window widgets and declare local variables
    GtkWidget * spinbutton4 = lookup_widget(propertybox1, "spinbutton4");

    // Activate 'Apply' button and 'Save' callback in Prefs Dialog.
    gnome_property_box_changed((GnomePropertyBox*)propertybox1);

    // Set Preferences Value
    nslookup_retries = gtk_spin_button_get_value_as_int((GtkSpinButton *) spinbutton4);
}


void
on_spinbutton5_changed  (GtkEditable *editable, gpointer user_data)
{
    // Get pointers to window widgets and declare local variables
    GtkWidget * spinbutton5 = lookup_widget(propertybox1, "spinbutton5");

    // Activate 'Apply' button and 'Save' callback in Prefs Dialog.
    gnome_property_box_changed((GnomePropertyBox*)propertybox1);

    // Set Preferences Value
    nslookup_timeout = gtk_spin_button_get_value_as_int((GtkSpinButton *) spinbutton5);
}


void
on_optionmenu_toolbar_clicked   (GtkButton *button, gpointer user_data)
{
    // NOTE: This function is actually called from: on_propertybox1_apply
    // The GtkOptionMenu itself cannot trigger this function!!! (12/28/2001)

    // Get pointers to window widgets and declare local variables
    GtkWidget * optionmenu_toolbar = lookup_widget(propertybox1, "optionmenu_toolbar");
    GtkWidget * menu         = GTK_OPTION_MENU(optionmenu_toolbar)->menu;
    GtkWidget * active_item  = gtk_menu_get_active(GTK_MENU(menu));
    gint        active_index = g_list_index(GTK_MENU_SHELL(menu)->children, active_item);

    // Activate 'Apply' button and 'Save' callback in Prefs Dialog.
    gnome_property_box_changed((GnomePropertyBox*)propertybox1);

    // Set Preferences Value
    toolbar_setting_int = active_index;
}
